@@ -66,9 +66,10 @@
 #include <X11/extensions/shape.h>
 #include <X11/Xresource.h>
 #endif /* SHAPE */
-#include "module.h"
+#include "module_interface.h"
 #include "session.h"
-
+#include "stack.h"
+#include "move_resize.h"
 
 /* Used to parse command line of clients for specific desk requests. */
 /* Todo: check for multiple desks. */
@@ -128,7 +129,8 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
   XTextProperty text_prop;
   extern Boolean PPosOverride;
 
-  int do_shade, do_maximize;
+  int do_shade = 0;
+  int do_maximize = 0;
   int x_max, y_max, w_max, h_max;
 
   NeedToResizeToo = False;
@@ -231,6 +233,7 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
 
   /* get merged styles */
   lookup_style(tmp_win, &style);
+
   sflags = SGET_FLAGS_POINTER(style);
 
   /* copy iconboxes ptr (if any) */
@@ -256,7 +259,7 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
       tmp_win->fl = &Scr.DefaultDecor;
 #endif
 
-  tmp_win->title_height = GetDecor(tmp_win,TitleHeight);
+  tmp_win->title_g.height = GetDecor(tmp_win,TitleHeight);
 
   GetMwmHints(tmp_win);
   GetOlHints(tmp_win);
@@ -264,11 +267,6 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
   SelectDecor(tmp_win, sflags, SGET_BORDER_WIDTH(style),
 	      SGET_HANDLE_WIDTH(style));
 
-#ifdef GNOME
-  /* set GNOME window hints & FVWM flags translated from those hints */
-  GNOME_GetHints(tmp_win);
-#endif
-
 #ifdef SHAPE
   /* set boundary width to zero for shaped windows */
   if (tmp_win->wShaped)
@@ -277,17 +275,34 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
 
   memcpy(&(tmp_win->flags), sflags, sizeof(common_flags_type));
   /* find a suitable icon pixmap */
-  if(SHAS_ICON(sflags))
+  
+  if((tmp_win->wmhints) && (tmp_win->wmhints->flags & IconWindowHint))
+    {
+      if (SHAS_ICON(sflags) && (SICON_OVERRIDE(sflags) == ICON_OVERRIDE))
+        {
+          tmp_win->icon_bitmap_file = SGET_ICON_NAME(style);
+        }
+      else 
+        {
+          tmp_win->icon_bitmap_file = NULL;
+        }
+    }
+  else if((tmp_win->wmhints) && (tmp_win->wmhints->flags & IconPixmapHint))
+    {
+      if (SHAS_ICON(sflags) && (SICON_OVERRIDE(sflags) != NO_ICON_OVERRIDE))
+        {
+          tmp_win->icon_bitmap_file = SGET_ICON_NAME(style);
+        }
+      else 
+        {
+          tmp_win->icon_bitmap_file = NULL;
+        }
+    }
+  else if(SHAS_ICON(sflags))
     {
       /* an icon was specified */
       tmp_win->icon_bitmap_file = SGET_ICON_NAME(style);
     }
-  else if((tmp_win->wmhints)
-	  &&(tmp_win->wmhints->flags & (IconWindowHint|IconPixmapHint)))
-    {
-      /* window has its own icon */
-      tmp_win->icon_bitmap_file = NULL;
-    }
   else
     {
       /* use default icon */
@@ -306,12 +321,12 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
   GetWindowSizeHints (tmp_win);
 
   /* Tentative size estimate */
-  tmp_win->frame_width = tmp_win->attr.width+2*tmp_win->boundary_width;
-  tmp_win->frame_height = tmp_win->attr.height + tmp_win->title_height+
+  tmp_win->frame_g.width = tmp_win->attr.width+2*tmp_win->boundary_width;
+  tmp_win->frame_g.height = tmp_win->attr.height + tmp_win->title_g.height+
     2*tmp_win->boundary_width;
 
-  ConstrainSize(tmp_win, &tmp_win->frame_width, &tmp_win->frame_height, False,
-		0, 0);
+  ConstrainSize(tmp_win, &tmp_win->frame_g.width, &tmp_win->frame_g.height,
+		False, 0, 0);
 
   /* Find out if the client requested a specific desk on the command line. */
   /*  RBW - 11/20/1998 - allow a desk of -1 to work.  */
@@ -446,8 +461,8 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
     tmp_win->prev = tmp_win->next;
     tmp_win->next = tmp_win->next->next;
   }
-  /* tmp_win->prev points to the last window in the list, tmp_win->next is NULL.
-     Now fix the last window to point to tmp_win */
+  /* tmp_win->prev points to the last window in the list, tmp_win->next is
+   * NULL. Now fix the last window to point to tmp_win */
   tmp_win->prev->next = tmp_win;
   /*
       RBW - 11/13/1998 - add it into the stacking order chain also.
@@ -464,21 +479,22 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
       Thus it is important have this call *after* PlaceWindow and the
       stacking order initialization.
   */
-  MatchWinToSM(tmp_win, &x_max, &y_max, &w_max, &h_max, &do_shade, &do_maximize);
+  MatchWinToSM(tmp_win, &x_max, &y_max, &w_max, &h_max, &do_shade,
+	       &do_maximize);
 
   /* set up geometry */
-  tmp_win->frame_x = tmp_win->attr.x + tmp_win->old_bw;
-  tmp_win->frame_y = tmp_win->attr.y + tmp_win->old_bw;
-  tmp_win->frame_width = tmp_win->attr.width+2*tmp_win->boundary_width;
-  tmp_win->frame_height = tmp_win->attr.height + tmp_win->title_height
+  tmp_win->frame_g.x = tmp_win->attr.x + tmp_win->old_bw;
+  tmp_win->frame_g.y = tmp_win->attr.y + tmp_win->old_bw;
+  tmp_win->frame_g.width = tmp_win->attr.width+2*tmp_win->boundary_width;
+  tmp_win->frame_g.height = tmp_win->attr.height + tmp_win->title_g.height
 			  + 2 * tmp_win->boundary_width;
-  ConstrainSize(tmp_win, &tmp_win->frame_width, &tmp_win->frame_height, False,
-		0, 0);
-  tmp_win->title_x = tmp_win->title_y = 0;
+  ConstrainSize(tmp_win, &tmp_win->frame_g.width, &tmp_win->frame_g.height,
+		False, 0, 0);
+  tmp_win->title_g.x = tmp_win->title_g.y = 0;
   tmp_win->title_w = 0;
-  tmp_win->title_width = tmp_win->frame_width - 2 * tmp_win->boundary_width;
-  if(tmp_win->title_width < 1)
-    tmp_win->title_width = 1;
+  tmp_win->title_g.width = tmp_win->frame_g.width - 2*tmp_win->boundary_width;
+  if(tmp_win->title_g.width < 1)
+    tmp_win->title_g.width = 1;
 
   /* create windows */
 
@@ -517,9 +533,9 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
 #endif
 
   /* create the frame window, child of root, grandparent of client */
-  tmp_win->frame = XCreateWindow(dpy, Scr.Root, tmp_win->frame_x,
-				 tmp_win->frame_y, tmp_win->frame_width,
-				 tmp_win->frame_height, 0, Scr.depth,
+  tmp_win->frame = XCreateWindow(dpy, Scr.Root, tmp_win->frame_g.x,
+				 tmp_win->frame_g.y, tmp_win->frame_g.width,
+				 tmp_win->frame_g.height, 0, Scr.depth,
 				 InputOutput, Scr.viz, valuemask, &attributes);
 
 #if defined(PIXMAP_BUTTONS) && defined(BORDERSTYLE)
@@ -534,14 +550,17 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
 			   | EnterWindowMask | LeaveWindowMask | ExposureMask);
 
   if (HAS_TITLE(tmp_win)) {
-    tmp_win->title_x = tmp_win->boundary_width + tmp_win->title_height + 1;
-    tmp_win->title_y = tmp_win->boundary_width;
+    tmp_win->title_g.x = tmp_win->boundary_width + tmp_win->title_g.height + 1;
+    tmp_win->title_g.y = tmp_win->boundary_width;
     attributes.cursor = Scr.FvwmCursors[TITLE_CURSOR];
-    tmp_win->title_w = XCreateWindow (dpy, tmp_win->frame, tmp_win->title_x,
-				      tmp_win->title_y, tmp_win->title_width,
-				      tmp_win->title_height, 0, CopyFromParent,
-				      InputOutput, CopyFromParent, valuemask,
-				      &attributes);
+    tmp_win->title_w = XCreateWindow(dpy, tmp_win->frame,
+					tmp_win->title_g.x,
+					tmp_win->title_g.y,
+					tmp_win->title_g.width,
+					tmp_win->title_g.height, 0,
+					CopyFromParent,
+					InputOutput, CopyFromParent, valuemask,
+					&attributes);
     attributes.cursor = Scr.FvwmCursors[SYS];
     for(i = 4; i >= 0; i--) {
       if((i < Scr.nr_left_buttons) && (tmp_win->left_w[i] > 0)) {
@@ -558,9 +577,9 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
         }
 #endif
         tmp_win->left_w[i] = XCreateWindow (dpy, tmp_win->frame,
-					    tmp_win->title_height * i, 0,
-					    tmp_win->title_height,
-					    tmp_win->title_height, 0,
+					    tmp_win->title_g.height * i, 0,
+					    tmp_win->title_g.height,
+					    tmp_win->title_g.height, 0,
 					    CopyFromParent, InputOutput,
 					    CopyFromParent, valuemask,
 					    &attributes);
@@ -581,10 +600,10 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
         }
 #endif
         tmp_win->right_w[i] = XCreateWindow (dpy, tmp_win->frame,
-					     tmp_win->title_width
-					     - tmp_win->title_height * (i + 1),
-					     0, tmp_win->title_height,
-					     tmp_win->title_height, 0,
+					     tmp_win->title_g.width
+					     - tmp_win->title_g.height * (i+1),
+					     0, tmp_win->title_g.height,
+					     tmp_win->title_g.height, 0,
 					     CopyFromParent, InputOutput,
 					     CopyFromParent, valuemask,
 					     &attributes);
@@ -631,14 +650,16 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
   attributes.colormap = Scr.cmap;
   attributes.background_pixmap = None;
   attributes.event_mask = SubstructureRedirectMask;
-  tmp_win->Parent = XCreateWindow (dpy, tmp_win->frame, tmp_win->boundary_width,
+  tmp_win->Parent = XCreateWindow (dpy, tmp_win->frame,
+				   tmp_win->boundary_width,
 				   tmp_win->boundary_width
-				   + tmp_win->title_height,
-				   (tmp_win->frame_width
+				   + tmp_win->title_g.height,
+				   (tmp_win->frame_g.width
 				   - 2 * tmp_win->boundary_width),
-				   (tmp_win->frame_height
+				   (tmp_win->frame_g.height
 				   - 2 * tmp_win->boundary_width
-				   - tmp_win->title_height), 0, CopyFromParent,
+				   - tmp_win->title_g.height), 0,
+				   CopyFromParent,
 				   InputOutput, CopyFromParent, valuemask,
 				   &attributes);
 
@@ -646,7 +667,8 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
 #ifdef MINI_ICONS
   if (tmp_win->mini_pixmap_file) {
     tmp_win->mini_icon = CachePicture (dpy, Scr.NoFocusWin, NULL,
-				       tmp_win->mini_pixmap_file, Scr.ColorLimit);
+				       tmp_win->mini_pixmap_file,
+				       Scr.ColorLimit);
   }
   else {
     tmp_win->mini_icon = NULL;
@@ -675,12 +697,12 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
    * again in HandleMapNotify.
    */
   SET_MAPPED(tmp_win, 0);
-  width = tmp_win->frame_width;
-  tmp_win->frame_width = 0;
-  height = tmp_win->frame_height;
-  tmp_win->frame_height = 0;
+  width = tmp_win->frame_g.width;
+  tmp_win->frame_g.width = 0;
+  height = tmp_win->frame_g.height;
+  tmp_win->frame_g.height = 0;
 
-  SetupFrame(tmp_win, tmp_win->frame_x, tmp_win->frame_y,width,height,
+  SetupFrame(tmp_win, tmp_win->frame_g.x, tmp_win->frame_g.y,width,height,
 	     True, False);
 
   if (do_maximize) {
@@ -691,10 +713,10 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
     SetupFrame(tmp_win, x_max, y_max, w_max, h_max, TRUE, False);
     SetBorder(tmp_win, Scr.Hilite == tmp_win, True, True, None);
     /* fix orig values to not change page on unmaximize  */
-    if (tmp_win->orig_x >= Scr.MyDisplayWidth)
-      tmp_win->orig_x = tmp_win->orig_x % Scr.MyDisplayWidth;
-    if (tmp_win->orig_y >= Scr.MyDisplayHeight)
-      tmp_win->orig_y = tmp_win->orig_y % Scr.MyDisplayHeight;
+    if (tmp_win->orig_g.x >= Scr.MyDisplayWidth)
+      tmp_win->orig_g.x = tmp_win->orig_g.x % Scr.MyDisplayWidth;
+    if (tmp_win->orig_g.y >= Scr.MyDisplayHeight)
+      tmp_win->orig_g.y = tmp_win->orig_g.y % Scr.MyDisplayHeight;
   }
 
   if (do_shade) {
@@ -814,14 +836,14 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
     {
       XWarpPointer(dpy, Scr.Root, Scr.Root, 0, 0, Scr.MyDisplayWidth,
 		   Scr.MyDisplayHeight,
-		   tmp_win->frame_x + (tmp_win->frame_width>>1),
-		   tmp_win->frame_y + (tmp_win->frame_height>>1));
+		   tmp_win->frame_g.x + (tmp_win->frame_g.width>>1),
+		   tmp_win->frame_g.y + (tmp_win->frame_g.height>>1));
       Event.xany.type = ButtonPress;
       Event.xbutton.button = 1;
-      Event.xbutton.x_root = tmp_win->frame_x + (tmp_win->frame_width>>1);
-      Event.xbutton.y_root = tmp_win->frame_y + (tmp_win->frame_height>>1);
-      Event.xbutton.x = (tmp_win->frame_width>>1);
-      Event.xbutton.y = (tmp_win->frame_height>>1);
+      Event.xbutton.x_root = tmp_win->frame_g.x + (tmp_win->frame_g.width>>1);
+      Event.xbutton.y_root = tmp_win->frame_g.y + (tmp_win->frame_g.height>>1);
+      Event.xbutton.x = (tmp_win->frame_g.width>>1);
+      Event.xbutton.y = (tmp_win->frame_g.height>>1);
       Event.xbutton.subwindow = None;
       Event.xany.window = tmp_win->w;
       resize_window(&Event , tmp_win->w, tmp_win, C_WINDOW, "", 0);
@@ -831,6 +853,8 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
 #ifdef GNOME
   /* set GNOME hints on the window from flags set on tmp_win */
   GNOME_SetHints(tmp_win);
+  GNOME_SetLayer(tmp_win);
+  GNOME_SetDesk(tmp_win);
 #endif
 
   return (tmp_win);
@@ -892,6 +916,8 @@ void FetchWmProtocols (FvwmWindow *tmp)
   return;
 }
 
+
+
 /***********************************************************************
  *
  *  Procedure:
@@ -901,19 +927,30 @@ void FetchWmProtocols (FvwmWindow *tmp)
  *	tmp - the fvwm window structure to use
  *
  ***********************************************************************/
+
 void GetWindowSizeHints(FvwmWindow *tmp)
 {
   long supplied = 0;
+  Bool broken_hints = False;
 
-  if (!XGetWMNormalHints (dpy, tmp->w, &tmp->hints, &supplied))
+  if (HAS_OVERRIDE_SIZE_HINTS(tmp) ||
+      !XGetWMNormalHints (dpy, tmp->w, &tmp->hints, &supplied))
     tmp->hints.flags = 0;
 
   /* Beat up our copy of the hints, so that all important field are
    * filled in! */
   if (tmp->hints.flags & PResizeInc)
     {
-      if (tmp->hints.width_inc == 0) tmp->hints.width_inc = 1;
-      if (tmp->hints.height_inc == 0) tmp->hints.height_inc = 1;
+      if (tmp->hints.width_inc <= 0)
+        {
+          tmp->hints.width_inc = 1;
+	  broken_hints = True;
+        }
+      if (tmp->hints.height_inc <= 0)
+        {
+          tmp->hints.height_inc = 1;
+	  broken_hints = True;
+        }
     }
   else
     {
@@ -921,12 +958,90 @@ void GetWindowSizeHints(FvwmWindow *tmp)
       tmp->hints.height_inc = 1;
     }
 
+  if(tmp->hints.flags & PMinSize)
+    {
+      if (tmp->hints.min_width <= 0)
+        {
+	  if (tmp->hints.min_width < 0)
+	    broken_hints = True;
+          tmp->hints.min_width = 1;
+        }
+      if (tmp->hints.min_height <= 0)
+        {
+	  if (tmp->hints.min_height < 0)
+	    broken_hints = True;
+          tmp->hints.min_height = 1;
+        }
+    }
+  else
+    {
+      if(tmp->hints.flags & PBaseSize)
+	{
+	  tmp->hints.min_width = tmp->hints.base_width;
+	  tmp->hints.min_height = tmp->hints.base_height;
+	}
+      else
+	{
+	  tmp->hints.min_width = 1;
+	  tmp->hints.min_height = 1;
+	}
+    }
+
+  if(tmp->hints.flags & PMaxSize)
+    {
+      if(tmp->hints.max_width < tmp->hints.min_width)
+        {
+	  tmp->hints.max_width = MAX_WINDOW_WIDTH;
+          broken_hints = True;
+        }
+      if(tmp->hints.max_height < tmp->hints.min_height)
+        {
+	  tmp->hints.max_height = MAX_WINDOW_HEIGHT;
+          broken_hints = True;
+        }
+    }
+  else
+    {
+      tmp->hints.max_width = MAX_WINDOW_WIDTH;
+      tmp->hints.max_height = MAX_WINDOW_HEIGHT;
+    }
+
   /*
    * ICCCM says that PMinSize is the default if no PBaseSize is given,
    * and vice-versa.
    */
 
-  if(!(tmp->hints.flags & PBaseSize))
+  if(tmp->hints.flags & PBaseSize)
+    {
+      if (tmp->hints.base_width < 0)
+        {
+          tmp->hints.base_width = 0;
+          broken_hints = True;
+        }
+      if (tmp->hints.base_height < 0)
+        {
+          tmp->hints.base_height = 0;
+          broken_hints = True;
+        }
+      if ((tmp->hints.base_width > tmp->hints.min_width) ||
+          (tmp->hints.base_height > tmp->hints.min_height))
+        {
+          /* In this case, doing the aspect ratio calculation
+	     for window_size - base_size as prescribed by the
+             ICCCM is going to fail.
+             Resetting the flag disables the use of base_size 
+	     in aspect ratio calculation while it is still used 
+	     for grid sizing.
+           */
+          tmp->hints.flags &= ~PBaseSize;
+#if 0
+ 	/* Keep silent about this, since the Xlib manual actually
+  	   recommends making min <= base <= max ! */
+          broken_hints = True;
+#endif
+        }
+    }
+  else
     {
       if(tmp->hints.flags & PMinSize)
 	{
@@ -939,47 +1054,10 @@ void GetWindowSizeHints(FvwmWindow *tmp)
 	  tmp->hints.base_height = 0;
 	}
     }
-  if(!(tmp->hints.flags & PMinSize))
-    {
-      tmp->hints.min_width = tmp->hints.base_width;
-      tmp->hints.min_height = tmp->hints.base_height;
-    }
-  if(!(tmp->hints.flags & PMaxSize))
-    {
-      tmp->hints.max_width = MAX_WINDOW_WIDTH;
-      tmp->hints.max_height = MAX_WINDOW_HEIGHT;
-    }
-  if (tmp->hints.flags & PBaseSize)
-    {
-      if (((tmp->hints.flags & PMinSize) &&
-	   (tmp->hints.base_width < tmp->hints.min_width ||
-	    tmp->hints.base_height < tmp->hints.min_height)) ||
-	  ((tmp->hints.flags & PMaxSize) &&
-	   (tmp->hints.base_width > tmp->hints.max_width ||
-	    tmp->hints.base_height > tmp->hints.max_height)))
-	{
-	  /* The size hints are broken. Ignore min and max hints! */
-	  tmp->hints.max_width = MAX_WINDOW_WIDTH;
-	  tmp->hints.max_height = MAX_WINDOW_HEIGHT;
-	  tmp->hints.min_height = 1;
-	  tmp->hints.min_width = 1;
-	}
-    }
-  if(tmp->hints.max_width < tmp->hints.min_width)
-    tmp->hints.max_width = MAX_WINDOW_WIDTH;
-  if(tmp->hints.max_height < tmp->hints.min_height)
-    tmp->hints.max_height = MAX_WINDOW_HEIGHT;
-
-  /* Zero width/height windows are bad news! */
-  if(tmp->hints.min_height <= 0)
-    tmp->hints.min_height = 1;
-  if(tmp->hints.min_width <= 0)
-    tmp->hints.min_width = 1;
 
   if(!(tmp->hints.flags & PWinGravity))
     {
       tmp->hints.win_gravity = NorthWestGravity;
-      tmp->hints.flags |= PWinGravity;
     }
 
   if (tmp->hints.flags & PAspect)
@@ -991,6 +1069,8 @@ void GetWindowSizeHints(FvwmWindow *tmp)
 #define maxAspectY tmp->hints.max_aspect.y
 #define minAspectX tmp->hints.min_aspect.x
 #define minAspectY tmp->hints.min_aspect.y
+
+
     /*
     ** The math looks like this:
     **
@@ -1006,13 +1086,65 @@ void GetWindowSizeHints(FvwmWindow *tmp)
     ** seems safest.
     **
     */
-    if ((minAspectX * maxAspectY) > (maxAspectX * minAspectY))
-    {
-      tmp->hints.flags &= ~PAspect;
-      fvwm_msg(WARN,
-               "GetWindowSizeHints",
-               "window id 0x%08x max_aspect ratio is < min_aspect ratio -> ignoring, but program displaying this window should be fixed!!!!",
-               tmp->w);
-    }
+
+    /* We also ignore people who put negative entries into
+     * their aspect ratios. They deserve it. 
+     *  
+     * We cast to double here, since the values may be large. 
+     */
+    if ((maxAspectX < 0) || (maxAspectY < 0) ||
+        (minAspectX < 0) || (minAspectY < 0) || 
+        (((double)minAspectX * (double)maxAspectY) > 
+         ((double)maxAspectX * (double)minAspectY)))
+     {
+        broken_hints = True;      
+        tmp->hints.flags &= ~PAspect;
+        fvwm_msg (WARN, "GetWindowSizeHints",
+                 "%s window %#lx has broken aspect ratio: %d/%d - %d/%d\n",
+                  tmp->name, tmp->w, minAspectX, minAspectY, maxAspectX,
+	  	maxAspectY);
+     }
+    else 
+     {
+       /* protect against overflow */ 
+       if ((maxAspectX > 65536) || (maxAspectY > 65536))
+         {
+            double ratio = (double) maxAspectX / (double) maxAspectY;
+            if (ratio > 1.0)
+              {
+                 maxAspectX = 65536;
+                 maxAspectY = 65536 / ratio;
+              }
+            else
+              {
+                 maxAspectX = 65536 * ratio;
+                 maxAspectY = 65536;
+              } 
+         }  
+       if ((minAspectX > 65536) || (minAspectY > 65536))
+         {
+            double ratio = (double) minAspectX / (double) minAspectY;
+            if (ratio > 1.0)
+              {
+                 minAspectX = 65536;
+                 minAspectY = 65536 / ratio;
+              }
+            else
+              {
+                 minAspectX = 65536 * ratio;
+                 minAspectY = 65536;
+              } 
+         }  
+     }
   }
+
+  if (broken_hints)
+    {
+      fvwm_msg (WARN, "GetWindowSizeHints",
+               "%s window %#lx has broken size hints\n"
+               "Please report this to fvwm-workers@fvwm.org\n",
+                tmp->name, tmp->w);
+    }
 }
+
+
