@@ -69,6 +69,7 @@
 #endif
 
 #include "fvwm.h"
+#include "events.h"
 #include "icons.h"
 #include <X11/Xatom.h>
 #include "misc.h"
@@ -78,15 +79,16 @@
 #ifdef SHAPE
 #include <X11/extensions/shape.h>
 #endif /* SHAPE */
-#include "module.h"
+#include "module_interface.h"
 #include "session.h"
+#include "focus.h"
+#include "stack.h"
+#include "move_resize.h"
 
 #ifndef XUrgencyHint
 #define XUrgencyHint            (1L << 8)
 #endif
 
-extern int menuFromFrameOrWindowOrTitlebar;
-
 extern Boolean debugging;
 extern Bool fFvwmInStartup;
 
@@ -120,8 +122,6 @@ Window PressedW;
 #endif /* !LASTEvent */
 typedef void (*PFEH)(void);
 PFEH EventHandlerJumpTable[LASTEvent];
-void  ResyncFvwmStackRing(void);
-void BroadcastRestack(FvwmWindow *,FvwmWindow *);
 
 /*
 ** Procedure:
@@ -462,7 +462,7 @@ void HandleKeyPress(void)
 void HandlePropertyNotify(void)
 {
   XTextProperty text_prop;
-  Boolean       OnThisPage    =  False;
+  Bool OnThisPage = False;
   int old_wmhints_flags;
 
   DBUG("HandlePropertyNotify","Routine Entered");
@@ -476,16 +476,7 @@ void HandlePropertyNotify(void)
       Make sure at least part of window is on this page
       before giving it focus...
   */
-  if ( (Tmp_win->Desk == Scr.CurrentDesk) &&
-       ( ((Tmp_win->frame_x + Tmp_win->frame_width) >= 0 &&
-          Tmp_win->frame_x < Scr.MyDisplayWidth) &&
-         ((Tmp_win->frame_y + Tmp_win->frame_height) >= 0 &&
-          Tmp_win->frame_y < Scr.MyDisplayHeight)
-       )
-     )
-    {
-      OnThisPage  =  True;
-    }
+  OnThisPage = IsRectangleOnThisPage(&(Tmp_win->frame_g), Tmp_win->Desk);
 
   switch (Event.xproperty.atom)
     {
@@ -653,12 +644,12 @@ void HandlePropertyNotify(void)
       */
       {
 	int new_width, new_height;
-	new_width = Tmp_win->frame_width;
-	new_height = Tmp_win->frame_height;
+	new_width = Tmp_win->frame_g.width;
+	new_height = Tmp_win->frame_g.height;
 	ConstrainSize(Tmp_win, &new_width, &new_height, False, 0, 0);
-	if((new_width != Tmp_win->frame_width)||
-	   (new_height != Tmp_win->frame_height))
-	  SetupFrame(Tmp_win,Tmp_win->frame_x, Tmp_win->frame_y,
+	if((new_width != Tmp_win->frame_g.width)||
+	   (new_height != Tmp_win->frame_g.height))
+	  SetupFrame(Tmp_win,Tmp_win->frame_g.x, Tmp_win->frame_g.y,
 		     new_width,new_height,False,False);
       }
 #endif /* 0 */
@@ -825,7 +816,7 @@ void HandleMapRequestKeepRaised(Window KeepRaised,  FvwmWindow  *ReuseWin)
 {
   extern long isIconicState;
   extern Boolean PPosOverride;
-  Boolean       OnThisPage    =  False;
+  Bool OnThisPage = False;
 
   Event.xany.window = Event.xmaprequest.window;
 
@@ -856,16 +847,7 @@ void HandleMapRequestKeepRaised(Window KeepRaised,  FvwmWindow  *ReuseWin)
       Make sure at least part of window is on this page
       before giving it focus...
   */
-  if ( (Tmp_win->Desk == Scr.CurrentDesk) &&
-       ( ((Tmp_win->frame_x + Tmp_win->frame_width) >= 0 &&
-          Tmp_win->frame_x < Scr.MyDisplayWidth) &&
-         ((Tmp_win->frame_y + Tmp_win->frame_height) >= 0 &&
-          Tmp_win->frame_y < Scr.MyDisplayHeight)
-       )
-     )
-    {
-      OnThisPage  =  True;
-    }
+  OnThisPage = IsRectangleOnThisPage(&(Tmp_win->frame_g), Tmp_win->Desk);
 
   if(KeepRaised != None)
     XRaiseWindow(dpy,KeepRaised);
@@ -898,9 +880,9 @@ void HandleMapRequestKeepRaised(Window KeepRaised,  FvwmWindow  *ReuseWin)
 	      XMapWindow(dpy, Tmp_win->frame);
 	      SET_MAP_PENDING(Tmp_win, 1);
 	      SetMapStateProp(Tmp_win, NormalState);
-	      if((!Scr.Focus) ||
-		 (!IS_TRANSIENT(Tmp_win) && DO_GRAB_FOCUS(Tmp_win)) ||
-		 (IS_TRANSIENT(Tmp_win) && DO_GRAB_FOCUS_TRANSIENT(Tmp_win)))
+	      if((!IS_TRANSIENT(Tmp_win) && DO_GRAB_FOCUS(Tmp_win)) ||
+		 (IS_TRANSIENT(Tmp_win) && DO_GRAB_FOCUS_TRANSIENT(Tmp_win) &&
+		  Scr.Focus && Scr.Focus->w == Tmp_win->transientfor))
 		{
                   if (OnThisPage && !HAS_NEVER_FOCUS(Tmp_win))
                     {
@@ -957,7 +939,7 @@ void HandleMapRequestKeepRaised(Window KeepRaised,  FvwmWindow  *ReuseWin)
  ***********************************************************************/
 void HandleMapNotify(void)
 {
-  Boolean       OnThisPage    =  False;
+  Bool OnThisPage = False;
 
   DBUG("HandleMapNotify","Routine Entered");
 
@@ -973,7 +955,7 @@ void HandleMapNotify(void)
     }
 
   /* Except for identifying over-ride redirect window mappings, we
-   * don't need or want windows associated with the sunstructurenotifymask */
+   * don't need or want windows associated with the substructurenotifymask */
   if(Event.xmap.event != Event.xmap.window)
     return;
 
@@ -981,16 +963,7 @@ void HandleMapNotify(void)
       Make sure at least part of window is on this page
       before giving it focus...
   */
-  if ( (Tmp_win->Desk == Scr.CurrentDesk) &&
-       ( ((Tmp_win->frame_x + Tmp_win->frame_width) >= 0 &&
-          Tmp_win->frame_x < Scr.MyDisplayWidth) &&
-         ((Tmp_win->frame_y + Tmp_win->frame_height) >= 0 &&
-          Tmp_win->frame_y < Scr.MyDisplayHeight)
-       )
-     )
-    {
-      OnThisPage  =  True;
-    }
+  OnThisPage = IsRectangleOnThisPage(&(Tmp_win->frame_g), Tmp_win->Desk);
 
   /*
    * Need to do the grab to avoid race condition of having server send
@@ -1017,9 +990,9 @@ void HandleMapNotify(void)
     BroadcastPacket(M_MAP, 3,
                     Tmp_win->w,Tmp_win->frame, (unsigned long)Tmp_win);
 
-  if((!Scr.Focus) ||
-     (!IS_TRANSIENT(Tmp_win) && DO_GRAB_FOCUS(Tmp_win)) ||
-     (IS_TRANSIENT(Tmp_win) && DO_GRAB_FOCUS_TRANSIENT(Tmp_win)))
+  if((!IS_TRANSIENT(Tmp_win) && DO_GRAB_FOCUS(Tmp_win)) ||
+     (IS_TRANSIENT(Tmp_win) && DO_GRAB_FOCUS_TRANSIENT(Tmp_win) &&
+      Scr.Focus && Scr.Focus->w == Tmp_win->transientfor))
     {
       if (OnThisPage && !HAS_NEVER_FOCUS(Tmp_win))
         {
@@ -1096,6 +1069,10 @@ void HandleUnmapNotify(void)
   if(Scr.PreviousFocus == Tmp_win)
     Scr.PreviousFocus = NULL;
 
+  if((!IS_TRANSIENT(Tmp_win) && DO_GRAB_FOCUS(Tmp_win)) ||
+     (IS_TRANSIENT(Tmp_win) && DO_GRAB_FOCUS_TRANSIENT(Tmp_win) &&
+      Scr.Focus && Scr.Focus->w == Tmp_win->transientfor))
+
   focus_grabbed = (Tmp_win == Scr.Focus) &&
     ((!IS_TRANSIENT(Tmp_win) && DO_GRAB_FOCUS(Tmp_win)) ||
      (IS_TRANSIENT(Tmp_win) && DO_GRAB_FOCUS_TRANSIENT(Tmp_win))) &&
@@ -1233,7 +1210,7 @@ void HandleButtonPress(void)
            (Event.xbutton.window == Tmp_win->frame) &&
 	   Scr.go.MouseFocusClickRaises)
   {
-    if (Tmp_win != Scr.LastWindowRaised &&
+    if (CanBeRaised(Tmp_win) &&
         MaskUsedModifiers(Event.xbutton.state) == 0 &&
         GetContext(Tmp_win,&Event, &PressedW) == C_WINDOW)
     {
@@ -1318,13 +1295,13 @@ void HandleEnterNotify(void)
     int delta_x=0, delta_y=0;
     /* this was in the HandleMotionNotify before, HEDU */
     HandlePaging(Scr.EdgeScrollX,Scr.EdgeScrollY,
-                 &Event.xcrossing.x_root,&Event.xcrossing.y_root,
+                 &ewp->x_root,&ewp->y_root,
                  &delta_x,&delta_y,True,True);
     return;
   }
 
   /* multi screen? */
-  if (Event.xany.window == Scr.Root)
+  if (ewp->window == Scr.Root)
   {
     if (!Scr.Focus || HAS_MOUSE_FOCUS(Scr.Focus))
     {
@@ -1342,9 +1319,9 @@ void HandleEnterNotify(void)
     return;
 
   if(HAS_MOUSE_FOCUS(Tmp_win) || HAS_SLOPPY_FOCUS(Tmp_win))
-    {
-      SetFocus(Tmp_win->w,Tmp_win,1);
-    }
+  {
+    SetFocus(Tmp_win->w,Tmp_win,1);
+  }
   if (Scr.ColormapFocus == COLORMAP_FOLLOWS_MOUSE)
   {
     if((!IS_ICONIFIED(Tmp_win))&&(Event.xany.window == Tmp_win->w))
@@ -1353,7 +1330,10 @@ void HandleEnterNotify(void)
       InstallWindowColormaps(NULL);
   }
 
-  if (IS_ICONIFIED(Tmp_win))
+  /* We get an EnterNotify with mode == UnGrab when fvwm releases
+     the grab held during iconification. We have to ignore this,
+     or icon title will be initially raised. */
+  if (IS_ICONIFIED(Tmp_win) && (ewp->mode == NotifyNormal))
     {
       SET_ICON_ENTERED(Tmp_win,1);
       DrawIconWindow (Tmp_win);
@@ -1404,105 +1384,6 @@ void HandleLeaveNotify(void)
 
 
 
-Bool
-intersect (int x0, int y0, int w0, int h0,
-           int x1, int y1, int w1, int h1)
-{
-  return !((x0 > x1 + w1) || (x0 + w0 < x1) ||
-           (y0 > y1 + h1) || (y0 + h0 < y1));
-}
-
-Bool
-overlap_box (FvwmWindow *r, int x, int y, int w, int h)
-{
-  if (IS_ICONIFIED(r))
-  {
-    return ((r->icon_pixmap_w) &&
-             intersect (x, y, w, h, r->icon_x_loc, r->icon_y_loc,
-                             r->icon_p_width, r->icon_p_height)) ||
-           ((r->icon_w) &&
-             intersect (x, y, w, h, r->icon_xl_loc, r->icon_y_loc + r->icon_p_height,
-                             r->icon_w_width, r->icon_w_height));
-  }
-  else
-  {
-    return intersect (x, y, w, h, r->frame_x, r->frame_y,
-                           r->frame_width, r->frame_height);
-  }
-}
-
-Bool
-overlap (FvwmWindow *r, FvwmWindow *s)
-{
-  if (r->Desk != s->Desk)
-  {
-    return 0;
-  }
-
-  if (IS_ICONIFIED(r))
-  {
-    return ((r->icon_pixmap_w) &&
-             overlap_box (s, r->icon_x_loc, r->icon_y_loc,
-                             r->icon_p_width, r->icon_p_height)) ||
-           ((r->icon_w) &&
-             overlap_box (s, r->icon_xl_loc, r->icon_y_loc + r->icon_p_height,
-                             r->icon_w_width, r->icon_w_height));
-  }
-  else
-  {
-    return overlap_box (s, r->frame_x, r->frame_y,
-                           r->frame_width, r->frame_height);
-  }
-}
-
-/* return true if stacking order changed */
-Bool
-HandleUnusualStackmodes(unsigned int stack_mode, FvwmWindow *r, Window rw,
-                                                 FvwmWindow *s, Window sw)
-{
-  Bool restack = 0;
-  FvwmWindow *t;
-
-/*  DBUG("HandleUnusualStackmodes", "called with %d, %lx\n", stack_mode, s);*/
-
-  if (((rw != r->w) ^ IS_ICONIFIED(r)) ||
-      (s && (((sw != s->w) ^ IS_ICONIFIED(s)) || (r->Desk != s->Desk))))
-  {
-    /* one of the relevant windows is unmapped */
-    return;
-  }
-
-  switch (stack_mode)
-  {
-   case TopIf:
-    for (t = r->stack_prev; (t != &Scr.FvwmRoot) && !restack; t = t->stack_prev)
-    {
-      restack = (((s == NULL) || (s == t)) && overlap (t, r));
-    }
-    if (restack)
-    {
-      RaiseWindow (r);
-    }
-    break;
-   case BottomIf:
-    for (t = r->stack_next; (t != &Scr.FvwmRoot) && !restack; t = t->stack_next)
-    {
-      restack = (((s == NULL) || (s == t)) && overlap (t, r));
-    }
-    if (restack)
-    {
-      LowerWindow (r);
-    }
-    break;
-   case Opposite:
-    restack = (HandleUnusualStackmodes (TopIf, r, rw, s, sw) ||
-               HandleUnusualStackmodes (BottomIf, r, rw, s, sw));
-    break;
-  }
-/*  DBUG("HandleUnusualStackmodes", "\t---> %d\n", restack);*/
-  return restack;
-}
-
 /***********************************************************************
  *
  *  Procedure:
@@ -1516,7 +1397,6 @@ void HandleConfigureRequest(void)
   int x, y, width, height;
   XConfigureRequestEvent *cre = &Event.xconfigurerequest;
   Bool sendEvent=False;
-  FvwmWindow  *FvwmSib;
 
   DBUG("HandleConfigureRequest","Routine Entered");
 
@@ -1605,10 +1485,10 @@ void HandleConfigureRequest(void)
  if (cre->window == Tmp_win->w)
  {
   /* Don't modify frame_XXX fields before calling SetupWindow! */
-  x = Tmp_win->frame_x;
-  y = Tmp_win->frame_y;
-  width = Tmp_win->frame_width;
-  height = Tmp_win->frame_height;
+  x = Tmp_win->frame_g.x;
+  y = Tmp_win->frame_g.y;
+  width = Tmp_win->frame_g.width;
+  height = Tmp_win->frame_g.height;
 
   /* for restoring */
   if (cre->value_mask & CWBorderWidth)
@@ -1620,11 +1500,11 @@ void HandleConfigureRequest(void)
   if (cre->value_mask & CWX)
     x = cre->x - Tmp_win->boundary_width;
   if (cre->value_mask & CWY)
-    y = cre->y - Tmp_win->boundary_width - Tmp_win->title_height;
+    y = cre->y - Tmp_win->boundary_width - Tmp_win->title_g.height;
   if (cre->value_mask & CWWidth)
     width = cre->width + 2*Tmp_win->boundary_width;
   if (cre->value_mask & CWHeight)
-    height = cre->height+Tmp_win->title_height+2*Tmp_win->boundary_width;
+    height = cre->height+Tmp_win->title_g.height+2*Tmp_win->boundary_width;
 
   /*
    * SetupWindow (x,y) are the location of the upper-left outer corner and
@@ -1637,8 +1517,9 @@ void HandleConfigureRequest(void)
   if (IS_SHADED(Tmp_win))
     {
       /* for shaded windows, allow resizing, but keep it shaded */
-      SetupFrame (Tmp_win, x, y, width, Tmp_win->frame_height,sendEvent,False);
-      Tmp_win->orig_ht = height;
+      SetupFrame (Tmp_win, x, y, width, Tmp_win->frame_g.height,sendEvent,
+		  False);
+      Tmp_win->orig_g.height = height;
     }
   else if (!IS_MAXIMIZED(Tmp_win))
     {
@@ -1770,7 +1651,7 @@ void HandleShapeNotify (void)
     if (sev->kind != ShapeBounding)
       return;
     Tmp_win->wShaped = sev->shaped;
-    SetShape(Tmp_win,Tmp_win->frame_width);
+    SetShape(Tmp_win,Tmp_win->frame_g.width);
   }
 }
 #endif  /* SHAPE*/
@@ -1809,7 +1690,8 @@ fd_set init_fdset;
 
 int My_XNextEvent(Display *dpy, XEvent *event)
 {
-  extern int fd_width, x_fd;
+  extern fd_set_size_t fd_width;
+  extern int x_fd;
   fd_set in_fdset, out_fdset;
   Window targetWindow;
   int i;
@@ -1862,11 +1744,11 @@ int My_XNextEvent(Display *dpy, XEvent *event)
 
   DBUG("My_XNextEvent","waiting for module input/output");
   XFlush(dpy);
-  if (select((SELECT_TYPE_ARG1)fd_width,
-             SELECT_TYPE_ARG234 &in_fdset,
-             SELECT_TYPE_ARG234 &out_fdset,
-             SELECT_TYPE_ARG234 0,
-             SELECT_TYPE_ARG5   timeoutP) > 0) {
+  if (select( fd_width,
+	      SELECT_FD_SET_CAST &in_fdset,
+	      SELECT_FD_SET_CAST &out_fdset,
+	      SELECT_FD_SET_CAST 0,
+              timeoutP) > 0) {
 
     /* Check for module input. */
     for (i=0; i<npipes; i++) {
@@ -1904,78 +1786,3 @@ int My_XNextEvent(Display *dpy, XEvent *event)
 }
 
 
-
-
-/*
-    RBW - 01/07/1998  -  this is here temporarily - I mean to move it to
-    libfvwm eventually, along with some other chain manipulation functions.
-*/
-
-/*
-    ResyncFvwmStackRing -
-    Rebuilds the stacking order ring of FVWM-managed windows. For use in cases
-    where apps raise/lower their own windows in a way that makes it difficult
-    to determine exactly where they ended up in the stacking order.
-    - Based on code from Matthias Clasen.
-*/
-void  ResyncFvwmStackRing (void)
-{
-  Window root, parent, *children;
-  unsigned int nchildren, i;
-  FvwmWindow *t1, *t2;
-
-  MyXGrabServer (dpy);
-
-  if (!XQueryTree (dpy, Scr.Root, &root, &parent, &children, &nchildren))
-    {
-      MyXUngrabServer (dpy);
-      return;
-    }
-
-  t2 = &Scr.FvwmRoot;
-  for (i = 0; i < nchildren; i++)
-    {
-      for (t1 = Scr.FvwmRoot.next; t1 != NULL; t1 = t1->next)
-	{
-          if (IS_ICONIFIED(t1) && !IS_ICON_SUPPRESSED(t1))
-            {
-	      if (t1->icon_w == children[i])
-	        {
-	          break;
-	        }
-              else if (t1->icon_pixmap_w == children[i])
-	        {
-	          break;
-	        }
-            }
-          else
-            {
-	      if (t1->frame == children[i])
-	        {
-	          break;
-	        }
-            }
-	}
-
-      if (t1 != NULL && t1 != t2)
-	{
-          /*
-              Move the window to its new position, working from the bottom up
-              (that's the way XQueryTree presents the list).
-          */
-          t1->stack_prev->stack_next = t1->stack_next;  /* Pluck from chain. */
-          t1->stack_next->stack_prev = t1->stack_prev;
-          t1->stack_next = t2;                          /* Set new pointers. */
-          t1->stack_prev = t2->stack_prev;
-          t2->stack_prev->stack_next = t1;              /* Insert in new
-							 * position. */
-          t2->stack_prev = t1;
-          t2 = t1;
-
-	}
-    }
-
-  MyXUngrabServer (dpy);
-
-  XFree (children);
-}
