@@ -37,6 +37,9 @@
 #include "focus.h"
 #include "icons.h"
 #include "screen.h"
+#include "bindings.h"
+#include "stack.h"
+
 
 static Bool lastFocusType;
 
@@ -133,7 +136,8 @@ static void DoSetFocus(Window w, FvwmWindow *Fw, Bool FocusByMouse,
 	}
     }
 
-  if((Fw != NULL)&&(!IsWindowOnThisPage(Fw))&&(!NoWarp))
+  if((Fw != NULL)&&
+     (!IsRectangleOnThisPage(&(Fw->frame_g),Fw->Desk))&&(!NoWarp))
     {
       Fw = NULL;
       w = Scr.NoFocusWin;
@@ -247,8 +251,8 @@ void FocusOn(FvwmWindow *t, Bool FocusByMouse, char *action)
     }
     else
     {
-      cx = t->frame_x + t->frame_width/2;
-      cy = t->frame_y + t->frame_height/2;
+      cx = t->frame_g.x + t->frame_g.width/2;
+      cy = t->frame_g.y + t->frame_g.height/2;
     }
 
     dx = (cx + Scr.Vx)/Scr.MyDisplayWidth*Scr.MyDisplayWidth;
@@ -257,10 +261,11 @@ void FocusOn(FvwmWindow *t, Bool FocusByMouse, char *action)
     MoveViewport(dx,dy,True);
 
     /* If the window is still not visible, make it visible! */
-    if(((t->frame_x + t->frame_height)< 0)||(t->frame_y + t->frame_width < 0)||
-       (t->frame_x >Scr.MyDisplayWidth)||(t->frame_y>Scr.MyDisplayHeight))
+    if(((t->frame_g.x + t->frame_g.height)< 0)||
+       (t->frame_g.y + t->frame_g.width < 0)||
+       (t->frame_g.x >Scr.MyDisplayWidth)||(t->frame_g.y>Scr.MyDisplayHeight))
     {
-      SetupFrame(t,0,0,t->frame_width, t->frame_height,False,False);
+      SetupFrame(t,0,0,t->frame_g.width, t->frame_g.height,False,False);
       if(HAS_MOUSE_FOCUS(t) || HAS_SLOPPY_FOCUS(t))
 	XWarpPointer(dpy, None, Scr.Root, 0, 0, 0, 0, 2,2);
     }
@@ -296,8 +301,8 @@ void WarpOn(FvwmWindow *t,int warp_x, int x_unit, int warp_y, int y_unit)
   }
   else
   {
-    cx = t->frame_x + t->frame_width/2;
-    cy = t->frame_y + t->frame_height/2;
+    cx = t->frame_g.x + t->frame_g.width/2;
+    cy = t->frame_g.y + t->frame_g.height/2;
   }
 
   dx = (cx + Scr.Vx)/Scr.MyDisplayWidth*Scr.MyDisplayWidth;
@@ -313,13 +318,13 @@ void WarpOn(FvwmWindow *t,int warp_x, int x_unit, int warp_y, int y_unit)
   else
   {
     if (x_unit != Scr.MyDisplayWidth)
-      x = t->frame_x + warp_x;
+      x = t->frame_g.x + warp_x;
     else
-      x = t->frame_x + (t->frame_width - 1) * warp_x / 100;
+      x = t->frame_g.x + (t->frame_g.width - 1) * warp_x / 100;
     if (y_unit != Scr.MyDisplayHeight)
-      y = t->frame_y + warp_y;
+      y = t->frame_g.y + warp_y;
     else
-      y = t->frame_y + (t->frame_height - 1) * warp_y / 100;
+      y = t->frame_g.y + (t->frame_g.height - 1) * warp_y / 100;
   }
   if (warp_x >= 0 && warp_y >= 0) {
     XWarpPointer(dpy, None, Scr.Root, 0, 0, 0, 0, x, y);
@@ -327,10 +332,11 @@ void WarpOn(FvwmWindow *t,int warp_x, int x_unit, int warp_y, int y_unit)
   RaiseWindow(t);
 
   /* If the window is still not visible, make it visible! */
-  if(((t->frame_x + t->frame_height)< 0)||(t->frame_y + t->frame_width < 0)||
-     (t->frame_x >Scr.MyDisplayWidth)||(t->frame_y>Scr.MyDisplayHeight))
+  if(((t->frame_g.x + t->frame_g.height)< 0)||
+     (t->frame_g.y + t->frame_g.width < 0)||
+     (t->frame_g.x >Scr.MyDisplayWidth)||(t->frame_g.y>Scr.MyDisplayHeight))
   {
-    SetupFrame(t,0,0,t->frame_width, t->frame_height,False,False);
+    SetupFrame(t,0,0,t->frame_g.width, t->frame_g.height,False,False);
     XWarpPointer(dpy, None, Scr.Root, 0, 0, 0, 0, 2,2);
   }
   UngrabEm();
