@@ -37,6 +37,7 @@
 
 #include <X11/keysym.h>
 #include "fvwm.h"
+#include "events.h"
 #include "style.h"
 #include "functions.h"
 #include "menus.h"
@@ -46,15 +47,16 @@
 #include "screen.h"
 #include "bindings.h"
 #include "repeat.h"
-#include "module.h"
-
+#include "Module.h"
+#include "move_resize.h"
 
 extern XEvent Event;
 extern FvwmWindow *Tmp_win;
 
 /* forward declarations */
 static void ComplexFunction(F_CMD_ARGS);
-static void ComplexFunction2(F_CMD_ARGS, Bool *desperate);
+static void execute_complex_function(F_CMD_ARGS, Bool *desperate,
+				     expand_command_type expand_cmd);
 
 /*
  * be sure to keep this list properly ordered for bsearch routine!
@@ -116,6 +118,7 @@ static const struct functions func_config[] =
   {"current",      CurrentFunc,      F_CURRENT,             0},
   {"cursormove",   movecursor,       F_MOVECURSOR,          0},
   {"cursorstyle",  CursorStyle,      F_CURSOR_STYLE,        0},
+  {"defaultbackground",SetDefaultBackground, F_DFLT_BACK,         0},
   {"defaultcolors",SetDefaultColors, F_DFLT_COLORS,         0},
   {"defaultfont",  LoadDefaultFont,  F_DFLT_FONT,           0},
   {"defaulticon",  SetDefaultIcon,   F_DFLT_ICON,           0},
@@ -165,6 +168,7 @@ static const struct functions func_config[] =
   {"modulepath",   setModulePath,    F_MODULE_PATH,         0},
   {"mouse",        mouse_binding,    F_MOUSE,               0},
   {"move",         move_window,      F_MOVE,                FUNC_NEEDS_WINDOW},
+  {"movethreshold",SetMoveThreshold, F_MOVE_THRESHOLD,      0},
   {"movetodesk",   move_window_to_desk,F_MOVE_TO_DESK,      FUNC_NEEDS_WINDOW},
   {"movetopage",   move_window_to_page,F_MOVE_TO_PAGE,      FUNC_NEEDS_WINDOW},
   {"next",         NextFunc,         F_NEXT,                0},
@@ -380,16 +384,18 @@ static Bool IsClick(int x,int y,unsigned EndMask, XEvent *d, Bool may_time_out)
 {
   int xcurrent,ycurrent,total = 0;
   Time t0;
+  int dist;
   extern Time lastTimestamp;
 
   xcurrent = x;
   ycurrent = y;
   t0 = lastTimestamp;
+  dist = Scr.MoveThreshold;
 
   while(((total < Scr.ClickTime && lastTimestamp - t0 < Scr.ClickTime) ||
 	 !may_time_out) &&
-	x - xcurrent < 3 && x - xcurrent > -3 &&
-	y - ycurrent < 3 && y - ycurrent > -3)
+	x - xcurrent < dist && x - xcurrent > -dist &&
+	y - ycurrent < dist && y - ycurrent > -dist)
     {
       usleep(20000);
       total+=20;
@@ -525,13 +531,14 @@ void ExecuteFunction(char *Action, FvwmWindow *tmp_win, XEvent *eventp,
     {
       Bool desperate = 1;
 
-      ComplexFunction2(eventp,w,tmp_win,context,taction, &Module, &desperate);
+      execute_complex_function(eventp,w,tmp_win,context,taction, &Module,
+			       &desperate, DONT_EXPAND_COMMAND);
       if(desperate)
 	  executeModule(eventp,w,tmp_win,context,taction, &Module);
     }
 
   /* Only wait for an all-buttons-up condition after calls from
-   * regular built-ins, not from complex-functions or modules. */
+   * regular built-ins, not from complex-functions, menus or modules. */
   if(Module == -1)
     WaitForButtonsUp();
 
@@ -540,9 +547,9 @@ void ExecuteFunction(char *Action, FvwmWindow *tmp_win, XEvent *eventp,
     Scr.flags.silent_functions = 0;
 
   if (must_free_string)
-{
+  {
     free(expaction);
-}
+  }
 
   func_depth--;
   return;
@@ -874,10 +881,12 @@ static void ComplexFunction(F_CMD_ARGS)
 {
   Bool desperate = 0;
 
-  ComplexFunction2(eventp, w, tmp_win, context, action, Module, &desperate);
+  execute_complex_function(eventp, w, tmp_win, context, action, Module,
+			   &desperate, EXPAND_COMMAND);
 }
 
-static void ComplexFunction2(F_CMD_ARGS, Bool *desperate)
+static void execute_complex_function(F_CMD_ARGS, Bool *desperate,
+				     expand_command_type expand_cmd)
 {
   static unsigned int cfunc_depth = 0;
   char type = MOTION;
@@ -1018,7 +1027,7 @@ static void ComplexFunction2(F_CMD_ARGS, Bool *desperate)
 	  else
 	    w = None;
 	  taction = expand(fi->action,arguments,tmp_win);
-	  ExecuteFunction(taction,tmp_win,ev,context,-2,EXPAND_COMMAND);
+	  ExecuteFunction(taction,tmp_win,ev,context,-2,expand_cmd);
 	  free(taction);
 	}
       fi = fi->next_item;
