@@ -1,3 +1,18 @@
+/* This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
+
 #ifndef _FUNCTIONS_
 #define _FUNCTIONS_
 
@@ -5,12 +20,12 @@ struct FvwmFunction;               /* forward declaration */
 
 typedef struct FunctionItem
 {
-  struct FvwmFunction *func;       /* the menu this item is in */
+  struct FvwmFunction *func;       /* the function this item is in */
   struct FunctionItem *next_item;  /* next function item */
   char condition;                  /* the character string displayed on left*/
   char *action;                    /* action to be performed */
   short type;                      /* type of built in function */
-  Bool f_needs_window;
+  unsigned char flags;
 } FunctionItem;
 
 typedef struct FvwmFunction
@@ -32,11 +47,12 @@ struct functions
   void (*action)();
 #endif
   short func_type;
-  Bool func_needs_window;
+  unsigned char flags;
 };
 
-#define FUNC_NO_WINDOW False
-#define FUNC_NEEDS_WINDOW True
+/* Bits for the function flag byte. */
+#define FUNC_NEEDS_WINDOW 0x01
+#define FUNC_DONT_REPEAT  0x02
 
 /* Types of events for the FUNCTION builtin */
 #define MOTION                'm'
@@ -51,7 +67,7 @@ typedef enum
   EXPAND_COMMAND
 } expand_command_type;
 
-void find_func_type(char *action, short *func_type, Bool *func_needs_window);
+void find_func_type(char *action, short *func_type, unsigned char *flags);
 FvwmFunction *FindFunction(char *function_name);
 extern FvwmFunction *NewFvwmFunction(char *name);
 void DestroyFunction(FvwmFunction *func);
