@@ -33,17 +33,23 @@
 #include <fcntl.h>
 #include <string.h>
 #include <unistd.h>
-#include <pwd.h>
+
+#ifdef HAVE_GETPWUID
+#  include <pwd.h>
+#endif
 
 #include "fvwm.h"
+#include "events.h"
 #include "functions.h"
 #include "menus.h"
 #include "misc.h"
 #include "screen.h"
 #include "parse.h"
-#include "module.h"
+#include "Module.h"
+#include "read.h"
 #include "session.h"
 
+
 #include <X11/Xproto.h>
 #include <X11/Xatom.h>
 /* need to get prototype for XrmUniqueQuark for XUniqueContext call */
@@ -75,16 +81,10 @@ Window BlackoutWin=None;        /* window to hide window captures */
 Bool fFvwmInStartup = True;     /* Set to False when startup has finished */
 Bool DoingCommandLine = True;	/* False after starting all cmd line modules */
 
-char *default_config_command = "Read "FVWMRC;
-
 #define MAX_CFG_CMDS 10
 static char *config_commands[MAX_CFG_CMDS];
 static int num_config_commands=0;
 
-#if 0
-/* unsused */
-char *output_file = NULL;
-#endif
 
 int FvwmErrorHandler(Display *, XErrorEvent *);
 int CatchFatal(Display *);
@@ -137,7 +137,7 @@ extern XEvent Event;
 Bool Restarting = False;
 int fd_width, x_fd;
 char *display_name = NULL;
-char *user_home_ptr;
+char *user_home_dir;
 
 typedef enum { FVWM_RUNNING=0, FVWM_DONE, FVWM_RESTART } FVWM_STATE;
 
@@ -175,7 +175,6 @@ int main(int argc, char **argv)
   int visualClass = -1;
   int visualId = -1;
   int x, y;
-  char *home_ptr;
 
   g_argv = argv;
   g_argc = argc;
@@ -187,29 +186,19 @@ int main(int argc, char **argv)
   putenv("FVWM_MODULEDIR=" FVWM_MODULEDIR);
 
   /* Figure out where to read and write config files. */
-  home_ptr = getenv("HOME");
-  if (home_ptr == NULL) {
-    home_ptr=".";
-  }
-  user_home_ptr = getenv("FVWM_USERHOME");
-  if (user_home_ptr != NULL) {
-    if (user_home_ptr[0] != '/') {
-      char *work_ptr;
-      work_ptr = safemalloc(strlen(home_ptr) + strlen(user_home_ptr) + 3);
-      strcpy (work_ptr, home_ptr);
-      strcat (work_ptr, "/");
-      strcat (work_ptr, user_home_ptr);
-      /* must have trailing slash like $HOME does. */
-      if (work_ptr[strlen(work_ptr)-1] != '/') {
-        strcat(work_ptr,"/");
-      }
-      user_home_ptr = work_ptr;
-    }
-  } else {
-    user_home_ptr = safemalloc(strlen(home_ptr) + 2);
-    strcpy(user_home_ptr,home_ptr);
-    strcat(user_home_ptr,"/");
+  user_home_dir = getenv("FVWM_USERHOME");
+  if ( user_home_dir == NULL )
+    user_home_dir = getenv("HOME");
+#ifdef HAVE_GETPWUID
+  if ( user_home_dir == NULL ) {
+    struct passwd* pw = getpwuid(getuid());
+    if ( pw != NULL )
+      user_home_dir = strdup( pw->pw_dir );
   }
+#endif
+  if ( user_home_dir == NULL )
+    user_home_dir = "."; /* give up and use current dir */
+
   for (i = 1; i < argc; i++)
   {
     if (strncasecmp(argv[i],"-debug",6)==0)
@@ -271,15 +260,6 @@ int main(int argc, char **argv)
         fvwm_msg(ERR,"main","only %d -f and -cmd parms allowed!",MAX_CFG_CMDS);
       }
     }
-#if 0
-/* unused */
-    else if (strncasecmp(argv[i],"-outfile",8)==0)
-    {
-      if (++i >= argc)
-        usage();
-      output_file = argv[i];
-    }
-#endif
     else if (strncasecmp(argv[i],"-h",2)==0)
     {
       usage();
@@ -487,6 +467,9 @@ int main(int argc, char **argv)
     exit(1);
   }
 
+  /* create the Scr.bg structure (required for the following visual stuff) */
+  Scr.bg = (Background *)safemalloc(sizeof(Background));
+
   if (visualClass != -1) {
     /* grab the best visual of the required class */
     XVisualInfo template, *vizinfo;
@@ -536,10 +519,12 @@ int main(int argc, char **argv)
 
   /* use default visuals if none found so far */
   if (visualClass == -1 && visualId == -1) {
-      Scr.viz = DefaultVisual(dpy, Scr.screen);
-      Scr.depth = DefaultDepth(dpy, Scr.screen);
-      Scr.cmap = DefaultColormap(dpy, Scr.screen);
-  }
+    Scr.viz = DefaultVisual(dpy, Scr.screen);
+    Scr.depth = DefaultDepth(dpy, Scr.screen);
+    Scr.cmap = DefaultColormap(dpy, Scr.screen);
+    Scr.usingDefaultVisual = True;
+  } else
+    Scr.usingDefaultVisual = False;
 
 #ifdef SHAPE
   ShapesSupported=XShapeQueryExtension(dpy, &ShapeEventBase, &ShapeErrorBase);
@@ -622,11 +607,22 @@ int main(int argc, char **argv)
 		      EXPAND_COMMAND);
       free(config_commands[i]);
     }
-  }
-  else
-  {
-    ExecuteFunction(default_config_command, NULL,&Event,C_ROOT,-1,
-		    EXPAND_COMMAND);
+  } else {
+      /** Run startup commands in ~/.fvwm2rc or FVWM_CONFIGDIR/system.fvwm2rc.
+	  If these fail, try FVWM_CONFIGDIR/.fvwm2rc for backwards compat. **/
+      if ( !run_command_file( CatString3(user_home_dir, "/", FVWMRC),
+			      &Event, NULL, C_ROOT, -1 )
+	   && !run_command_file( CatString3(FVWM_CONFIGDIR, "/system", FVWMRC),
+				 &Event, NULL, C_ROOT, -1 )
+	   && !run_command_file( CatString3(FVWM_CONFIGDIR, "/", FVWMRC),
+				 &Event, NULL, C_ROOT, -1 ) )
+      {
+	  fvwm_msg( ERR, "main",
+	    "Cannot read startup file.  Tried %s/%s, %s/system%s, and %s/%s",
+		    user_home_dir, FVWMRC,
+		    FVWM_CONFIGDIR, FVWMRC,
+		    FVWM_CONFIGDIR, FVWMRC );
+      }
   }
 
   DoingCommandLine = False;
@@ -781,6 +777,7 @@ void CaptureOneWindow(FvwmWindow *fw, Window window)
     return;
   if(XFindContext(dpy, window, FvwmContext, (caddr_t *)&fw)!=XCNOENT)
   {
+    PPosOverride = True;
     isIconicState = DontCareState;
     if(XGetWindowProperty(dpy, fw->w, _XA_WM_STATE, 0L, 3L, False,
 			  _XA_WM_STATE, &atype, &aformat, &nitems,
@@ -805,6 +802,7 @@ void CaptureOneWindow(FvwmWindow *fw, Window window)
     Destroy(fw);
     Event.xmaprequest.window = w;
     HandleMapRequestKeepRaised(BlackoutWin, fw);
+    PPosOverride = False;
     return;
   }
 }
@@ -1502,6 +1500,10 @@ void InitVariables(void)
   Scr.StdShadowGC = 0;
   Scr.DrawGC = 0;
   Scr.DrawPicture = NULL;
+  Scr.bg->pixmap = (Pixmap)0;
+  Scr.bg->type.word = 0;
+  XQueryBestTile(dpy, Scr.NoFocusWin, 0, 0,
+		 &Scr.bestTileWidth, &Scr.bestTileHeight);
 
   /* zero all flags */
   memset(&Scr.flags, 0, sizeof(Scr.flags));
@@ -1580,6 +1582,7 @@ void InitVariables(void)
   Scr.SnapGridX = 1;
   Scr.SnapGridY = 1;
   Scr.OpaqueSize = 5;
+  Scr.MoveThreshold = DEFAULT_MOVE_THRESHOLD;
   /* ClickTime is set to the positive value upon entering the event loop. */
   Scr.ClickTime = -DEFAULT_CLICKTIME;
   Scr.ColormapFocus = COLORMAP_FOLLOWS_MOUSE;
@@ -1655,7 +1658,7 @@ void Reborder(void)
   }
 
   MyXUngrabServer (dpy);
-  XSetInputFocus (dpy, PointerRoot, RevertToPointerRoot,CurrentTime);
+  XSetInputFocus (dpy, PointerRoot, RevertToPointerRoot, CurrentTime);
   XSync(dpy,0);
 
 }
@@ -1693,14 +1696,7 @@ void Done(int restart, char *command)
 
   if(restart)
   {
-     struct passwd   *pwd;
-     char filename[1024];
-     FILE *cfg_file;
-
-     pwd = getpwuid (getuid());
-
-     strncpy (filename, pwd->pw_dir, 1024);
-     strncat (filename, "/.fvwm_restart", 1024);
+     char* filename = strdup( CatString2(user_home_dir, "/.fvwm_restart") );
 
      /* We currently still need this, since InitVariables
         sets the Restarting flag based on the presence of
@@ -1710,10 +1706,10 @@ void Done(int restart, char *command)
         deals with WM_DESKTOP could be eliminated. */
      SaveDesktopState();		/* I wonder why ... */
 
-     cfg_file = fopen (filename, "w");
-     SaveWindowStates (cfg_file);
-     SaveGlobalState (cfg_file);
-     fclose (cfg_file);
+     if (!command || strstr (command, "fvwm"))
+       {
+	 RestartInSession (filename); /* won't return under SM */
+       }
 
     /* Really make sure that the connection is closed and cleared! */
     XSelectInput(dpy, Scr.Root, 0 );
