@@ -59,10 +59,8 @@ unsigned long context,char *action, int *Module
 #include <X11/Xutil.h>
 #include <X11/cursorfont.h>
 #include <X11/Intrinsic.h>
-
 #include <libs/Picture.h>
-
-#include "gsfr.h"
+#include "window_flags.h"
 
 
 #ifndef WithdrawnState
@@ -93,11 +91,20 @@ unsigned long context,char *action, int *Module
 
 #define NULLSTR ((char *) NULL)
 
+typedef struct
+{
+  int x;
+  int y;
+  int width;
+  int height;
+} rectangle;
+
 /*
   For 1 style statement, there can be any number of IconBoxes.
   The name list points at the first one in the chain.
  */
-typedef struct icon_boxes_struct {
+typedef struct icon_boxes_struct
+{
   struct icon_boxes_struct *next;       /* next icon_boxes or zero */
   int IconBox[4];                       /* x/y x/y for iconbox */
   short IconGrid[2];                    /* x incr, y incr */
@@ -156,6 +163,8 @@ typedef struct
   unsigned is_lenient : 1;
   unsigned is_sticky : 1;
   unsigned ignore_restack : 1;
+  unsigned is_fixed : 1;
+  unsigned has_depressable_border : 1;
 } common_flags_type;
 
 typedef struct
@@ -214,6 +223,11 @@ typedef struct
   unsigned has_no_title : 1;
   unsigned has_ol_decor : 1;
   unsigned use_start_on_desk : 1;
+  unsigned icon_override : 2;
+#define NO_ACTIVE_ICON_OVERRIDE 0
+#define ICON_OVERRIDE           1
+#define NO_ICON_OVERRIDE        2
+#define ICON_OVERRIDE_MASK      3 
 } style_flags;
 
 /* only style.c and add_window.c are allowed to access this struct!! */
@@ -277,16 +291,28 @@ typedef struct FvwmWindow
 #ifdef SHAPE
     int wShaped;               /* is this a shaped window */
 #endif
+
+#if 1
+    rectangle frame_g;
+#else
     int frame_x;		/* x position of frame */
     int frame_y;		/* y position of frame */
     int frame_width;		/* width of frame */
     int frame_height;		/* height of frame */
+#endif
+
     int boundary_width;
     int corner_width;
+
+#if 1
+    rectangle title_g;
+#else
     int title_x;
     int title_y;
     int title_height;		/* height of the title bar */
     int title_width;		/* width of the title bar */
+#endif
+
     int icon_x_loc;		/* icon window x coordinate */
     int icon_xl_loc;		/* icon label window x coordinate */
     int icon_y_loc;		/* icon window y coordiante */
@@ -318,10 +344,14 @@ typedef struct FvwmWindow
 #endif
     char *icon_bitmap_file;
 
+#if 1
+    rectangle orig_g;
+#else
     int orig_x;                 /* unmaximized x coordinate */
     int orig_y;                 /* unmaximized y coordinate */
     int orig_wd;                /* unmaximized window width */
     int orig_ht;                /* unmaximized window height */
+#endif
 
     int maximized_ht;           /* maximized window height */
 
@@ -425,8 +455,8 @@ typedef struct WindowConditionMask {
 
 #include <stdlib.h>
 extern void Reborder(void);
-extern void SigDone(int);
-extern void Restart(int nonsense);
+RETSIGTYPE SigDone(int);
+RETSIGTYPE Restart(int nonsense);
 extern void Done(int, char *) __attribute__((__noreturn__));
 extern void BlackoutScreen(void);
 extern void UnBlackoutScreen(void);
@@ -452,7 +482,7 @@ extern Boolean ShapesSupported;
 extern Window JunkRoot, JunkChild;
 extern int JunkX, JunkY;
 extern unsigned int JunkWidth, JunkHeight, JunkBW, JunkDepth, JunkMask;
-extern char *user_home_ptr;
+extern char *user_home_dir;
 
 extern Atom _XA_MIT_PRIORITY_COLORS;
 extern Atom _XA_WM_CHANGE_STATE;
