@@ -66,6 +66,13 @@
 #include <libs/fvwmlib.h>
 #include "fvwm.h"
 
+
+
+/*************************
+ * MENU STYLE STRUCTURES *
+ *************************/
+
+
 typedef enum {
     /* menu types */
     SimpleMenu = 0,
@@ -86,11 +93,11 @@ typedef enum {
 typedef struct MenuFeel {
     struct
     {
-      unsigned Animated : 1;
-      unsigned PopupImmediately : 1;
-      unsigned TitleWarp : 1;
-      unsigned PopupAsRootmenu : 1;
-    } f;
+      unsigned is_animated : 1;
+      unsigned do_popup_immediately : 1;
+      unsigned do_title_warp : 1;
+      unsigned do_popup_as_root_menu : 1;
+    } flags;
     int PopupOffsetPercent;
     int PopupOffsetAdd;
 } MenuFeel;
@@ -116,14 +123,14 @@ typedef struct MenuLook {
     MenuFace face;
     struct
     {
-      unsigned Hilight : 1;
-      unsigned hasActiveFore : 1;
-      unsigned hasActiveBack : 1;
-      unsigned hasStippleFore : 1;
-      unsigned LongSeparators : 1;
-      unsigned TriangleRelief : 1;
-      unsigned hasSideColor : 1;
-    } f;
+      unsigned do_hilight : 1;
+      unsigned has_active_fore : 1;
+      unsigned has_active_back : 1;
+      unsigned has_stipple_fore : 1;
+      unsigned has_long_separators : 1;
+      unsigned has_triangle_relief : 1;
+      unsigned has_side_color : 1;
+    } flags;
     char ReliefThickness;
     char TitleUnderlines;
     Picture *sidePic;
@@ -149,28 +156,40 @@ typedef struct MenuStyle {
     MenuFeel feel;
 } MenuStyle;
 
+
+
+/************************
+ * MENU ITEM STRUCTURES *
+ ************************/
+
+
 struct MenuRoot; /* forward declaration */
 
+/* IMPORTANT NOTE: Don't put members into this struct that can change while the
+ * menu is visible! This will wreak havoc on recursive menus when they finally
+ * get implemented. */
 typedef struct MenuItem
 {
     struct MenuRoot *mr;        /* the menu this item is in */
     struct MenuItem *next;	/* next menu item */
     struct MenuItem *prev;	/* prev menu item */
+
     char *item;			/* the character string displayed on left*/
     char *item2;	        /* the character string displayed on right*/
+    short strlen;		/* strlen(item) */
+    short strlen2;		/* strlen(item2) */
+
     Picture *picture;           /* Pixmap to show  above label*/
     Picture *lpicture;          /* Pixmap to show to left of label */
-    char *action;		/* action to be performed */
-    short item_num;		/* item number of this menu */
+
     short x;			/* x coordinate for text (item) */
     short x2;			/* x coordinate for text (item2) */
     short xp;                   /* x coordinate for picture */
     short y_offset;		/* y coordinate for item */
     short y_height;		/* y height for item */
+
+    char *action;		/* action to be performed */
     short func_type;		/* type of built in function */
-    short state;		/* video state, 0 = normal, 1 = reversed */
-    short strlen;		/* strlen(item) */
-    short strlen2;		/* strlen(item2) */
     short hotkey;		/* Hot key offset (pete@tecc.co.uk).
 				   0 - No hot key
 				   +ve - offset to hot key char in item
@@ -179,11 +198,97 @@ typedef struct MenuItem
 				   refer to the *first* character)
 				   */
     char chHotkey;
-    short fIsSeparator;          /* 1 if this is a separator or a title */
+    struct
+    {
+      unsigned is_separator;
+      unsigned is_title;
+      unsigned is_popup;
+      unsigned is_menu;
+    } flags;
 } MenuItem;
 
+
+
+/************************
+ * MENU ROOT STRUCTURES *
+ ************************/
+
+/* This struct contains the parts of a root menu that are shared among all
+ * copies of the menu */
+typedef struct MenuRootStatic
+{
+    MenuItem *first;	/* first item in menu */
+    MenuItem *last;	/* last item in menu */
+
+    int copies;         /* # of copies, 0 if none except this one */
+
+    char *name;			/* name of root */
+    short height;		/* height of the menu */
+    short width0;               /* width of the menu-left-picture col */
+    short width;		/* width of the menu for 1st col */
+    short width2;		/* width of the menu for 2nd col */
+    short width3;               /* width of the submenu triangle col */
+    short xoffset;              /* the distance between the left border and the
+				 * beginning of the menu items */
+    short items;		/* number of items in the menu */
+    Picture *sidePic;
+    Pixel sideColor;
+    /* Menu Face    */
+    MenuStyle *ms;
+    /* temporary flags, deleted when menu pops down! */
+    struct
+    {
+      unsigned is_background_set : 1; /* is win background set? */
+      unsigned is_in_use : 1;
+      unsigned is_painted : 1;
+      unsigned is_left : 1;   /* menu direction relative to parent menu */
+      unsigned is_right : 1;
+      unsigned is_up : 1;
+      unsigned is_down : 1;
+    } tflags;
+    /* permanent flags */
+    struct
+    {
+      unsigned has_side_color : 1;
+    } flags;
+    struct
+    {
+      char *popup_action;
+      char *popdown_action;
+    } dynamic;
+} MenuRootStatic;
+
+
+/* This struct contains the parts of a root menu that differ in all copies of
+ * the menu */
+typedef struct MenuRootDynamic
+{
+    struct MenuRoot *original;  /* the first copy of the current menu */
+    struct MenuRoot *next;	/* next in list of root menus */
+    struct MenuRoot *continuation; /* continuation of this menu
+				    * (too tall for screen) */
+    /* can get the menu that this popped up through selected->mr when
+       selected is a popup menu item */
+    struct MenuRoot *mrDynamicPrev; /* the menu that popped this up, if any */
+    Window w;			/* the window of the menu */
+    MenuItem *selected;	        /* the selected item in menu */
+    int xanimation;             /* x distance window was moved by animation */
+#ifdef GRADIENT_BUTTONS
+    struct
+    {
+      Pixmap stored;
+      int width;
+      int height;
+      int y;
+    } stored_item;
+#endif
+} MenuRootDynamic;
+
+
+
 typedef struct MenuRoot
 {
+#if 1
     MenuItem *first;	/* first item in menu */
     MenuItem *last;	/* last item in menu */
     MenuItem *selected;	/* the selected item in menu */
@@ -207,35 +312,53 @@ typedef struct MenuRoot
     char *name;			/* name of root */
     Window w;			/* the window of the menu */
     short height;		/* height of the menu */
+    short width0;               /* width of the menu-left-picture col */
     short width;		/* width of the menu for 1st col */
     short width2;		/* width of the menu for 2nd col */
-    short width0;               /* width of the menu-left-picture col */
+    short width3;               /* width of the submenu triangle col */
+    short xoffset;              /* the distance between the left border and the
+				 * beginning of the menu items */
     short items;		/* number of items in the menu */
     Picture *sidePic;
     Pixel sideColor;
-    short xoffset;
-    MenuStyle *ms;        /* Menu Face    */
-    union                 /* internal flags, deleted when menu pops down! */
+    /* Menu Face    */
+    MenuStyle *ms;
+    /* temporary flags, deleted when menu pops down! */
+    struct
     {
-      /* need to change that type if we have more than 8 flags.
-       * more that a word will entail some changes in the code! */
-      unsigned char allflags;
-      struct
-      {
-	unsigned is_backgroundset : 1; /* is win background set? */
-	unsigned is_in_use : 1;
-	unsigned colorize : 1;
-	unsigned painted : 1;
-	unsigned is_left : 1;   /* menu direction relative to parent menu */
-	unsigned is_right : 1;
-	unsigned is_up : 1;
-	unsigned is_down : 1;
-      } f;
+      unsigned is_background_set : 1; /* is win background set? */
+      unsigned is_in_use : 1;
+      unsigned is_painted : 1;
+      unsigned is_left : 1;   /* menu direction relative to parent menu */
+      unsigned is_right : 1;
+      unsigned is_up : 1;
+      unsigned is_down : 1;
+    } tflags;
+    /* permanent flags */
+    struct
+    {
+      unsigned has_side_color : 1;
     } flags;
+    struct
+    {
+      char *popup_action;
+      char *popdown_action;
+    } dynamic;
     int xanimation;      /* x distance window was moved by animation     */
+#else
+    MenuRootStatic *s;
+    MenuRootDynamic *d;
+#endif
 } MenuRoot;
 /* don't forget to initialise new members in NewMenuRoot()! */
 
+
+
+/***********************************
+ * MENU OPTIONS AND POSITION HINTS *
+ ***********************************/
+
+
 typedef struct
 {
   int x;                  /* suggested x position */
@@ -248,20 +371,14 @@ typedef struct
 typedef struct
 {
   MenuPosHints pos_hints;
-  union
+  struct
   {
-    /* need to change that type if we have more than 8 flags.
-     * more that a word will entail some changes in the code! */
-    unsigned char allflags;
-    struct
-    {
-      unsigned no_warp : 1;
-      unsigned warp_title : 1;
-      unsigned fixed : 1;
-      unsigned select_in_place : 1;
-      unsigned select_warp : 1;
-      unsigned has_poshints : 1;
-    } f;
+    unsigned no_warp : 1;
+    unsigned warp_title : 1;
+    unsigned fixed : 1;
+    unsigned select_in_place : 1;
+    unsigned select_warp : 1;
+    unsigned has_poshints : 1;
   } flags;
 } MenuOptions;
 
@@ -269,6 +386,30 @@ extern MenuPosHints lastMenuPosHints;
 extern Bool fLastMenuPosHintsValid;
 
 
+
+/****************************
+ * MISCELLANEOUS MENU STUFF *
+ ****************************/
+
+typedef struct
+{
+  MenuRoot *menu;
+  MenuRoot *menu_prior;
+  FvwmWindow **pTmp_win;
+  FvwmWindow *button_window;
+  int *pcontext;
+  XEvent *eventp;
+  char **ret_paction;
+  MenuOptions *pops;
+  int cmenuDeep;
+  struct
+  {
+    unsigned is_menu_from_frame_or_window_or_titlebar : 1;
+    unsigned is_sticky : 1;
+  } flags;
+} MenuParameters;
+
+
 /* Return values for UpdateMenu, do_menu, menuShortcuts */
 /* Just uses enum-s for their constant value, replaced a bunch of #define-s
  * before */
@@ -286,7 +427,8 @@ typedef enum {
     MENU_POPUP,
     MENU_POPDOWN,
     MENU_SELECTED,
-    MENU_NEWITEM
+    MENU_NEWITEM,
+    MENU_TEAR_OFF
 } MenuStatus;
 
 
@@ -299,7 +441,7 @@ typedef struct MenuInfo
   int DoubleClickTime;
   struct
   {
-    unsigned animated_menus : 1;
+    unsigned use_animated_menus : 1;
   } flags;
 } MenuInfo;
 
@@ -310,6 +452,13 @@ extern MenuInfo Menus;
 #define MENU_ADD_BUTTON(x) ((x)==MENU_DONE || (x)==MENU_ABORTED?(x)+1:(x))
 #define MENU_ADD_BUTTON_IF(y,x) (y?MENU_ADD_BUTTON((x)):(x))
 
+
+
+/**********************
+ * EXPORTED FUNCTIONS *
+ **********************/
+
+
 MenuRoot *FollowMenuContinuations(MenuRoot *mr,MenuRoot **pmrPrior);
 void AnimatedMoveOfWindow(Window w,int startX,int startY,int endX, int endY,
 			  Bool fWarpPointerToo, int cusDelay,
@@ -318,13 +467,11 @@ MenuRoot *NewMenuRoot(char *name);
 void AddToMenu(MenuRoot *, char *, char *, Bool, Bool);
 void MakeMenu(MenuRoot *);
 Bool PopUpMenu(MenuRoot *, int, int);
-MenuStatus do_menu (MenuRoot *menu,MenuRoot *menuPrior,
-		    MenuItem **pmiExecuteAction, int cmenuDeep, Bool fSticks,
-		    XEvent *eventp, MenuOptions *pops);
+MenuStatus do_menu(MenuParameters *pmp);
 MenuRoot *FindPopup(char *popup_name);
 char *GetMenuOptions(char *action, Window w, FvwmWindow *tmp_win,
 		     MenuItem *mi, MenuOptions *pops);
-void DestroyMenu(MenuRoot *mr);
+void DestroyMenu(MenuRoot *mr, Bool recreate);
 void MakeMenus(void);
 void add_item_to_menu(F_CMD_ARGS);
 void add_another_menu_item(char *action);
