@@ -36,12 +36,15 @@
 #include <stdarg.h>
 
 #include "fvwm.h"
+#include "events.h"
 #include <X11/Xatom.h>
 #include <X11/Xutil.h>
+#include <X11/keysym.h>
 #include "misc.h"
 #include "parse.h"
 #include "screen.h"
-#include "module.h"
+#include "Module.h"
+#include "focus.h"
 
 FvwmWindow *FocusOnNextTimeStamp = NULL;
 
@@ -86,6 +89,7 @@ void Destroy(FvwmWindow *Tmp_win)
   extern FvwmWindow *ButtonWindow;
   extern FvwmWindow *colormap_win;
   extern Boolean PPosOverride;
+  Bool focus_set = False;
 
   /*
    * Warning, this is also called by HandleUnmapNotify; if it ever needs to
@@ -125,17 +129,34 @@ void Destroy(FvwmWindow *Tmp_win)
   if(ButtonWindow == Tmp_win)
     ButtonWindow = NULL;
 
-  if((Tmp_win == Scr.Focus)&&(HAS_CLICK_FOCUS(Tmp_win)))
+  if (Tmp_win->transientfor != None && Tmp_win->transientfor != Scr.Root)
     {
-      if(Tmp_win->next)
+      FvwmWindow *t;
+      for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
+      {
+	if (t->w == Tmp_win->transientfor)
+	  break;
+      }
+      if (t)
+      {
+	HandleHardFocus(t);
+	focus_set = True;
+      }
+    }
+  if (!focus_set)
+    {
+      if((Tmp_win == Scr.Focus)&&(HAS_CLICK_FOCUS(Tmp_win)))
 	{
-	  HandleHardFocus(Tmp_win->next);
+	  if(Tmp_win->next)
+	    {
+	      HandleHardFocus(Tmp_win->next);
+	    }
+	  else
+	    SetFocus(Scr.NoFocusWin, NULL,1);
 	}
-      else
+      else if(Scr.Focus == Tmp_win)
 	SetFocus(Scr.NoFocusWin, NULL,1);
     }
-  else if(Scr.Focus == Tmp_win)
-    SetFocus(Scr.NoFocusWin, NULL,1);
 
   if(Tmp_win == FocusOnNextTimeStamp)
     FocusOnNextTimeStamp = NULL;
@@ -298,12 +319,12 @@ void RestoreWithdrawnLocation (FvwmWindow *tmp,Bool restart)
       if(!restart)
 	{
 	  /* Don't mess with it if its partially on the screen now */
-	  if((tmp->frame_x < 0)||(tmp->frame_y<0)||
-	     (tmp->frame_x >= Scr.MyDisplayWidth)||
-	     (tmp->frame_y >= Scr.MyDisplayHeight))
+	  if((tmp->frame_g.x < 0)||(tmp->frame_g.y<0)||
+	     (tmp->frame_g.x >= Scr.MyDisplayWidth)||
+	     (tmp->frame_g.y >= Scr.MyDisplayHeight))
 	    {
-	      w2 = (tmp->frame_width>>1);
-	      h2 = (tmp->frame_height>>1);
+	      w2 = (tmp->frame_g.width>>1);
+	      h2 = (tmp->frame_g.height>>1);
 	      if (( xwc.x < -w2) || (xwc.x > (Scr.MyDisplayWidth-w2 )))
 		{
 		  xwc.x = xwc.x % Scr.MyDisplayWidth;
@@ -637,257 +658,6 @@ void MapIt(FvwmWindow *t)
    }
 }
 
-/* send RESTACK packets for all windows between s1 and s2 */
-void BroadcastRestack (FvwmWindow *s1, FvwmWindow *s2)
-{
-  FvwmWindow *t, *t2;
-  int num, i;
-  unsigned long *body, *bp, length;
-  extern Time lastTimestamp;
-
-  if (s1 == &Scr.FvwmRoot)
-   {
-      t = s1->stack_next;
-      /* t has been moved to the top of stack */
-
-      BroadcastPacket (M_RAISE_WINDOW, 3, t->w, t->frame, (unsigned long)t);
-      if (t->stack_next == s2)
-        {
-          /* avoid sending empty RESTACK packet */
-          return;
-        }
-   }
-  else
-   {
-      t = s1;
-   }
-  for (t2 = t, num = 1 ; t2 != s2; t2 = t2->stack_next, num++)
-    ;
-
-  length = HEADER_SIZE + 3*num;
-  body = (unsigned long *) safemalloc (length*sizeof(unsigned long));
-
-  bp = body;
-  *(bp++) = START_FLAG;
-  *(bp++) = M_RESTACK;
-  *(bp++) = length;
-  *(bp++) = lastTimestamp;
-  for (t2 = t; t2 != s2; t2 = t2->stack_next)
-   {
-      *(bp++) = t2->w;
-      *(bp++) = t2->frame;
-      *(bp++) = (unsigned long)t2;
-   }
-   for (i = 0; i < npipes; i++)
-     PositiveWrite(i, body, length*sizeof(unsigned long));
-
-   free (body);
-}
-
-/*
-   Raise t and its transients to the top of its layer.
-   For the pager to work properly it is necessary that
-   RaiseWindow *always* sends a proper M_RESTACK packet,
-   even if the stacking order didn't change.
-*/
-void RaiseWindow(FvwmWindow *t)
-{
-  FvwmWindow *s, *r, *t2, *next, tmp_r, tmp_s;
-  unsigned int flags;
-  int i, count;
-  XWindowChanges changes;
-  Window *wins;
-
-  Scr.LastWindowRaised = t;
-
-  /* detach t early, so it doesn't make trouble in the loops */
-  t->stack_prev->stack_next = t->stack_next;
-  t->stack_next->stack_prev = t->stack_prev;
-
-  count = 1;
-  if (IS_ICONIFIED(t) && !IS_ICON_SUPPRESSED(t))
-    {
-      count += 2;
-    }
-
-#ifndef DONT_RAISE_TRANSIENTS
-  /* collect the transients in a temp list */
-  tmp_s.stack_prev = &tmp_r;
-  tmp_r.stack_next = &tmp_s;
-  for (t2 = Scr.FvwmRoot.stack_next; t2 != &Scr.FvwmRoot; t2 = next)
-    {
-      next = t2->stack_next;
-      if ((IS_TRANSIENT(t2)) && (t2->transientfor == t->w) &&
-	  (t2->layer == t->layer))
-        {
-          /* t2 is a transient to raise */
-          count++;
-          if (IS_ICONIFIED(t2) && !IS_ICON_SUPPRESSED(t2))
-            {
-	      count += 2;
-            }
-
-          /* unplug it */
-          t2->stack_next->stack_prev = t2->stack_prev;
-          t2->stack_prev->stack_next = t2->stack_next;
-
-          /* put it above tmp_s */
-          t2->stack_next = &tmp_s;
-          t2->stack_prev = tmp_s.stack_prev;
-          t2->stack_next->stack_prev = t2;
-          t2->stack_prev->stack_next = t2;
-	}
-    }
-#endif /* DONT_RAISE_TRANSIENTS */
-
-  /* now find the place to reinsert t and friends */
-  for (s = Scr.FvwmRoot.stack_next; s != &Scr.FvwmRoot; s = s->stack_next)
-    {
-      if (t->layer >= s->layer)
-	{
-	  break;
-        }
-    }
-  r = s->stack_prev;
-
-#ifndef DONT_RAISE_TRANSIENTS
-  /* insert all transients between r and s. */
-  r->stack_next = tmp_r.stack_next;
-  r->stack_next->stack_prev = r;
-  s->stack_prev = tmp_s.stack_prev;
-  s->stack_prev->stack_next = s;
-#endif /* DONT_RAISE_TRANSIENTS */
-
-  /* don't forget t itself */
-  t->stack_next = s;
-  t->stack_prev = s->stack_prev;
-  t->stack_prev->stack_next = t;
-  t->stack_next->stack_prev = t;
-
-  wins = (Window*) safemalloc (count * sizeof (Window));
-
-  i = 0;
-  for (t2 = r->stack_next; t2 != s; t2 = t2->stack_next)
-    {
-      if (i >= count) {
-	fvwm_msg (ERR, "RaiseWindow", "more transients than expected");
-	break;
-      }
-      wins[i++] = t2->frame;
-      if (IS_ICONIFIED(t2) && !IS_ICON_SUPPRESSED(t2))
-	{
-	  if(t2->icon_w != None)
-	    wins[i++] = t2->icon_w;
-	  if(t2->icon_pixmap_w != None)
-	    wins[i++] = t2->icon_pixmap_w;
-	}
-    }
-
-  changes.sibling = s->frame;
-  if (changes.sibling != None)
-    {
-      changes.stack_mode = Above;
-      flags = CWSibling|CWStackMode;
-    }
-  else
-    {
-      changes.stack_mode = Below;
-      flags = CWStackMode;
-    }
-
-  XConfigureWindow (dpy, r->stack_next->frame, flags, &changes);
-  XRestackWindows (dpy, wins, count);
-
-  /* send out (one or more) M_RESTACK packets for windows between r and s */
-  BroadcastRestack (r, s);
-
-  free (wins);
-
-  raisePanFrames();
-
-  /*
-     The following is a hack to raise X windows over native windows
-     which is needed for some (all ?) X servers running under windows NT.
-   */
-  if (Scr.go.RaiseHackNeeded)
-    {
-      Window junk;
-      Window *tops;
-      int i, num;
-      Bool found = False;
-
-      /* get *all* toplevels (even including override_redirects) */
-      XQueryTree (dpy, Scr.Root, &junk, &junk, &tops, &num);
-
-      /* raise from tmp_win upwards to get them above NT windows */
-      for (i = 0; i < num; i++) {
-        if (tops[i] == t->frame) found = True;
-        if (found) XRaiseWindow (dpy, tops[i]);
-      }
-
-      XFree (tops);
-    }
-}
-
-
-void LowerWindow(FvwmWindow *t)
-{
-  FvwmWindow *s;
-  XWindowChanges changes;
-  unsigned int flags;
-
-  Scr.LastWindowRaised = NULL;
-
-  for (s = Scr.FvwmRoot.stack_next; s != &Scr.FvwmRoot; s = s->stack_next)
-    {
-      if (t->layer > s->layer)
-        {
-	  break;
-        }
-    }
-
-  if (s == t->stack_next)
-    {
-      return;
-    }
-
-
-  t->stack_prev->stack_next = t->stack_next;
-  t->stack_next->stack_prev = t->stack_prev;
-
-  t->stack_next = s;
-  t->stack_prev = s->stack_prev;
-  t->stack_prev->stack_next = t;
-  t->stack_next->stack_prev = t;
-
-  changes.sibling = s->frame;
-  if (changes.sibling != None)
-    {
-      changes.stack_mode = Above;
-      flags = CWSibling|CWStackMode;
-    }
-  else
-    {
-      changes.stack_mode = Below;
-      flags = CWStackMode;
-    }
-
-  XConfigureWindow(dpy, t->frame, flags, &changes);
-
-  if (IS_ICONIFIED(t) && !IS_ICON_SUPPRESSED(t))
-    {
-      changes.sibling = t->frame;
-      changes.stack_mode = Below;
-      flags = CWSibling|CWStackMode;
-      if (t->icon_w != None)
-	XConfigureWindow(dpy, t->icon_w, flags, &changes);
-      if (t->icon_pixmap_w != None)
-	XConfigureWindow(dpy, t->icon_pixmap_w, flags, &changes);
-    }
-
-  BroadcastRestack (t->stack_prev, t->stack_next);
-}
-
 
 void HandleHardFocus(FvwmWindow *t)
 {
@@ -1020,12 +790,138 @@ void ReapChildren(void)
 #endif
 }
 
-Bool IsWindowOnThisPage(FvwmWindow *fw)
+Bool IsRectangleOnThisPage(rectangle *rec, int desk)
 {
-  return (fw->Desk == Scr.CurrentDesk &&
-	  fw->frame_x > -fw->frame_width &&
-	  fw->frame_x < Scr.MyDisplayWidth &&
-	  fw->frame_y > -fw->frame_height &&
-	  fw->frame_y < Scr.MyDisplayHeight) ?
+  return (desk == Scr.CurrentDesk &&
+	  rec->x > -rec->width &&
+	  rec->x < Scr.MyDisplayWidth &&
+	  rec->y > -rec->height &&
+	  rec->y < Scr.MyDisplayHeight) ?
     True : False;
 }
+
+/****************************************************************************
+ *
+ * For menus, move, and resize operations, we can effect keyboard
+ * shortcuts by warping the pointer.
+ *
+ ****************************************************************************/
+void Keyboard_shortcuts(XEvent *Event, FvwmWindow *fw, int ReturnEvent)
+{
+  int x,y,x_root,y_root;
+  int x_move_size = 0, y_move_size = 0;
+  int x_move,y_move;
+
+  KeySym keysym;
+
+  if (fw)
+    {
+      x_move_size = fw->hints.width_inc;
+      y_move_size = fw->hints.height_inc;
+    }
+  if (y_move_size < 5)
+    y_move_size = 5;
+  if (x_move_size < 5)
+    x_move_size = 5;
+  if(Event->xkey.state & ControlMask)
+    x_move_size = y_move_size = 1;
+  if(Event->xkey.state & ShiftMask)
+    x_move_size = y_move_size = 100;
+
+  keysym = XLookupKeysym(&Event->xkey,0);
+
+  x_move = 0;
+  y_move = 0;
+  switch(keysym)
+    {
+    case XK_Up:
+    case XK_KP_8:
+    case XK_k:
+    case XK_p:
+      y_move = -y_move_size;
+      break;
+    case XK_Down:
+    case XK_KP_2:
+    case XK_n:
+    case XK_j:
+      y_move = y_move_size;
+      break;
+    case XK_Left:
+    case XK_KP_4:
+    case XK_b:
+    case XK_h:
+      x_move = -x_move_size;
+      break;
+    case XK_Right:
+    case XK_KP_6:
+    case XK_f:
+    case XK_l:
+      x_move = x_move_size;
+      break;
+    case XK_KP_1:
+      x_move = -x_move_size;
+      y_move = y_move_size;
+      break;
+    case XK_KP_3:
+      x_move = x_move_size;
+      y_move = y_move_size;
+      break;
+    case XK_KP_7:
+      x_move = -x_move_size;
+      y_move = -y_move_size;
+      break;
+    case XK_KP_9:
+      x_move = x_move_size;
+      y_move = -y_move_size;
+      break;
+    case XK_Return:
+    case XK_KP_Enter:
+    case XK_space:
+      /* beat up the event */
+      Event->type = ReturnEvent;
+      break;
+    case XK_Escape:
+      /* simple code to bag out of move - CKH */
+      /* return keypress event instead */
+      Event->type = KeyPress;
+      Event->xkey.keycode = XKeysymToKeycode(Event->xkey.display,keysym);
+      break;
+    default:
+      break;
+    }
+  XQueryPointer(dpy, Scr.Root, &JunkRoot, &Event->xany.window,
+		&x_root, &y_root, &x, &y, &JunkMask);
+
+  if((x_move != 0)||(y_move != 0))
+    {
+      /* beat up the event */
+      XWarpPointer(dpy, None, Scr.Root, 0, 0, 0, 0, x_root+x_move,
+		   y_root+y_move);
+
+      /* beat up the event */
+      Event->type = MotionNotify;
+      Event->xkey.x += x_move;
+      Event->xkey.y += y_move;
+      Event->xkey.x_root += x_move;
+      Event->xkey.y_root += y_move;
+    }
+}
+
+
+/****************************************************************************
+ *
+ * Check if the given FvwmWindow structure still points to a valid window.
+ *
+ ****************************************************************************/
+
+Bool check_if_fvwm_window_exists(FvwmWindow *fw)
+{
+  FvwmWindow *t;
+
+  for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
+  {
+    if (t == fw)
+      return True;
+  }
+  return False;
+}
