@@ -50,8 +50,8 @@
 #include "misc.h"
 #include "parse.h"
 #include "screen.h"
-#include "module.h"
-#include <libs/Picture.h>
+#include "module_interface.h"
+#include "libs/Picture.h"
 
 extern unsigned long *PipeMask;                /* in module.c */
 
@@ -178,7 +178,7 @@ void SendDataToModule(XEvent *eventp,Window w,FvwmWindow *tmp_win,
   char *ImagePath = GetImagePath();
 
   SendLook(*Module);
-  
+
   if (ImagePath && strlen(ImagePath))
   {
     message=safemalloc(strlen(ImagePath)+12);
@@ -197,7 +197,9 @@ void SendDataToModule(XEvent *eventp,Window w,FvwmWindow *tmp_win,
   sprintf(msg2,"ClickTime %d\n", (Scr.ClickTime < 0) ?
 	  -Scr.ClickTime : Scr.ClickTime);
   SendName(*Module,M_CONFIG_INFO,0,0,0,msg2);
-  
+  sprintf(msg2,"MoveThreshold %d\n", Scr.MoveThreshold);
+  SendName(*Module,M_CONFIG_INFO,0,0,0,msg2);
+
   t = modlistroot;
   while(t != NULL)
   {
@@ -210,19 +212,19 @@ void SendDataToModule(XEvent *eventp,Window w,FvwmWindow *tmp_win,
 
 char *make_look_packet()
 {
-  char *message = safemalloc(10 * (sizeof(long) * 2 + 1) + 18);
-  sprintf(message, "Default_graphics %lx %lx %x %lx %lx %lx %lx %lx %lx %lx\n",
+  char *b = safemalloc(DEFGRAPHNUM * (sizeof(long) * 2 + 1) + DEFGRAPHLEN + 3);
+  sprintf(b, "%s %lx %lx %lx %lx %lx %lx %lx %lx %lx\n",
+	  DEFGRAPHSTR,
  	  XVisualIDFromVisual(Scr.viz),
 	  Scr.cmap,
-	  Scr.depth,
 	  Scr.DrawGC ? XGContextFromGC(Scr.DrawGC) : 0,
-	  (long)0,
-	  Scr.StdColors.back,
+	  Scr.bg->type.word,
+	  Scr.bg->pixmap,
 	  Scr.StdGC ? XGContextFromGC(Scr.StdGC) : 0,
 	  Scr.StdReliefGC ? XGContextFromGC(Scr.StdReliefGC) : 0,
 	  Scr.StdShadowGC ? XGContextFromGC(Scr.StdShadowGC) : 0,
 	  Scr.StdFont.font ? Scr.StdFont.font->fid : 0);
-  return message;
+  return b;
 }
 
 void SendLook(int module)
