@@ -105,6 +105,7 @@ enum
   F_SNAP_ATT,
   F_SNAP_GRID,
   F_DFLT_FONT,
+  F_DFLT_BACK,
   F_DFLT_COLORS,
   F_DFLT_ICON,
   F_DFLT_LAYERS,
@@ -122,6 +123,7 @@ enum
   F_DESTROY,
   F_DELETE,
   F_MOVE,
+  F_MOVE_THRESHOLD,
   F_MOVE_TO_PAGE,
   F_ICONIFY,
   F_STICK,
