@@ -283,14 +283,22 @@ typedef struct ScreenInfo
   GC StdGC;
   GC StdReliefGC;
   GC StdShadowGC;
+  Bool usingDefaultVisual;
   Visual *viz;
   unsigned int depth;
   Colormap cmap;
+  unsigned int bestTileWidth;
+  unsigned int bestTileHeight;
+  /* this structure defines the default background, used by modules that have
+   * been converted to share fvwm's visuals. see libs/ModGraph.h */
+  Background *bg;
+
   GC DrawGC;			/* GC to draw lines for move and resize */
   Picture *DrawPicture;
   GC ScratchGC1;
   GC ScratchGC2;
   GC ScratchGC3;
+  GC IconPixmapGC;		/* ignored unless fvwm is using -visual */
   int SizeStringWidth;	        /* minimum width of size window */
   int BoundaryWidth;	        /* frame width for decorated windows */
   int NoBoundaryWidth;	        /* frame width for decorated windows */
@@ -317,8 +325,6 @@ typedef struct ScreenInfo
   int NumBoxes;
   int randomx;                  /* values used for randomPlacement */
   int randomy;
-  FvwmWindow *LastWindowRaised; /* Last window which was raised. Used for raise
-				 * lower func. */
   int VxMax;                    /* Max location for top left of virt desk*/
   int VyMax;
   int Vx;                       /* Current loc for top left of virt desk */
@@ -327,6 +333,8 @@ typedef struct ScreenInfo
   int ClickTime;               /*Max button-click delay for Function built-in*/
   int ScrollResistance;        /* resistance to scrolling in desktop */
   int MoveResistance;          /* res to moving windows over viewport edge */
+  int MoveThreshold;           /* number of pixels of mouse motion to decide
+				* it's a move operation */
   int SnapAttraction;          /* attractiveness of window edges */
   int SnapMode;                /* mode of snap attraction */
   int SnapGridX;               /* snap grid X size */
