@@ -224,6 +224,10 @@ void lookup_style(FvwmWindow *tmp_win, window_style *styles)
   memset(styles, 0, sizeof(window_style));
   /* initialize to default layer */
   styles->layer = Scr.DefaultLayer;
+#ifdef GNOME
+  /* initialize with GNOME hints */
+  GNOME_GetStyle (tmp_win, styles);
+#endif
   /* look thru all styles in order defined. */
   for (nptr = all_styles; nptr != NULL; nptr = nptr->next) {
     /* If name/res_class/res_name match, merge */
@@ -449,7 +453,13 @@ void ProcessNewStyle(XEvent *eventp, Window w, FvwmWindow *tmp_win,
         break;
 
       case 'd':
-        if(StrEquals(token, "DecorateTransient"))
+        if(StrEquals(token, "DepressableBorder"))
+	  {
+	  found = True;
+          tmpstyle.flags.common.has_depressable_border = 1;
+          tmpstyle.flag_mask.common.has_depressable_border = 1;
+	  }
+        else if(StrEquals(token, "DecorateTransient"))
         {
 	  found = True;
           tmpstyle.flags.do_decorate_transient = 1;
@@ -467,7 +477,19 @@ void ProcessNewStyle(XEvent *eventp, Window w, FvwmWindow *tmp_win,
         break;
 
       case 'f':
-        if(StrEquals(token, "FORECOLOR"))
+        if(StrEquals(token, "FirmBorder"))
+	  {
+	  found = True;
+          tmpstyle.flags.common.has_depressable_border = 0;
+          tmpstyle.flag_mask.common.has_depressable_border = 1;
+	  }
+        else if(StrEquals(token, "FixedPosition"))
+	  {
+	    found = True;
+            tmpstyle.flags.common.is_fixed = 1;
+            tmpstyle.flag_mask.common.is_fixed = 1;
+	  }
+        else if(StrEquals(token, "FORECOLOR"))
         {
 	  found = True;
 	  GetNextToken(rest, &token);
@@ -552,7 +574,13 @@ void ProcessNewStyle(XEvent *eventp, Window w, FvwmWindow *tmp_win,
         break;
 
       case 'i':
-        if(StrEquals(token, "IgnoreRestack"))
+	if(StrEquals(token, "IconOverride"))
+	{
+	  found = True;
+	  tmpstyle.flags.icon_override = ICON_OVERRIDE;
+	  tmpstyle.flag_mask.icon_override = ICON_OVERRIDE_MASK;
+	}
+        else if(StrEquals(token, "IgnoreRestack"))
         {
 	  found = True;
 	  tmpstyle.flags.common.ignore_restack = 1;
@@ -849,7 +877,19 @@ void ProcessNewStyle(XEvent *eventp, Window w, FvwmWindow *tmp_win,
         break;
 
       case 'n':
-        if(StrEquals(token, "NoIconTitle"))
+	if(StrEquals(token, "NoActiveIconOverride"))
+	{
+	  found = True;
+	  tmpstyle.flags.icon_override = NO_ACTIVE_ICON_OVERRIDE;
+	  tmpstyle.flag_mask.icon_override = ICON_OVERRIDE_MASK;
+	}
+	else if(StrEquals(token, "NoIconOverride"))
+	{
+	  found = True;
+	  tmpstyle.flags.icon_override = NO_ICON_OVERRIDE;
+	  tmpstyle.flag_mask.icon_override = ICON_OVERRIDE_MASK;
+	}
+        else if(StrEquals(token, "NoIconTitle"))
         {
 	  found = True;
           tmpstyle.flags.common.has_no_icon_title = 1;
@@ -1200,6 +1240,12 @@ void ProcessNewStyle(XEvent *eventp, Window w, FvwmWindow *tmp_win,
         break;
 
       case 'v':
+        if(StrEquals(token, "VariablePosition"))
+	  {
+	    found = True;
+            tmpstyle.flags.common.is_fixed = 0;
+            tmpstyle.flag_mask.common.is_fixed = 1;
+	  }
         break;
 
       case 'w':
