@@ -26,7 +26,9 @@
 #include "misc.h"
 #include "parse.h"
 #include "screen.h"
-#include "module.h"
+#include "Module.h"
+#include "focus.h"
+#include "move_resize.h"
 
 /*
  * dje 12/19/98
@@ -230,9 +232,8 @@ Bool HandlePaging(int HorWarpSize, int VertWarpSize, int *xl, int *yt,
 	     lastTimestamp - my_timestamp + add_time < Scr.ScrollResistance);
 
   if (lastTimestamp - my_timestamp + add_time < Scr.ScrollResistance)
-{
     return False;
-}
+
   /* Move the viewport */
   /* and/or move the cursor back to the approximate correct location */
   /* that is, the same place on the virtual desktop that it */
@@ -611,10 +612,10 @@ void MoveViewport(int newx, int newy, Bool grab)
 	    /*
 	        If the window is moving into the viewport...
 	    */
-            txl = t->frame_x;
-            tyt = t->frame_y;
-	    txr = t->frame_x + t->frame_width;
-	    tyb = t->frame_y + t->frame_height;
+            txl = t->frame_g.x;
+            tyt = t->frame_g.y;
+	    txr = t->frame_g.x + t->frame_g.width;
+	    tyb = t->frame_g.y + t->frame_g.height;
 	    if ((txr >= PageLeft && txl <= PageRight
 	        && tyb >= PageTop && tyt <= PageBottom)
 	        && !IS_VIEWPORT_MOVED(t)
@@ -647,8 +648,8 @@ void MoveViewport(int newx, int newy, Bool grab)
                                          t->icon_w_height+t->icon_p_height);
                         }
 		     }
-	           SetupFrame (t, t->frame_x+ deltax, t->frame_y + deltay,
-		          t->frame_width, t->frame_height,FALSE, False);
+	           SetupFrame (t, t->frame_g.x+ deltax, t->frame_g.y + deltay,
+		          t->frame_g.width, t->frame_g.height,FALSE, False);
 	         }
 	      }
             /*  Bump to next win...    */
@@ -659,10 +660,10 @@ void MoveViewport(int newx, int newy, Bool grab)
 	    /*
 	        If the window is not moving into the viewport...
 	    */
-            txl = t1->frame_x;
-            tyt = t1->frame_y;
-            txr = t1->frame_x + t1->frame_width;
-            tyb = t1->frame_y + t1->frame_height;
+            txl = t1->frame_g.x;
+            tyt = t1->frame_g.y;
+            txr = t1->frame_g.x + t1->frame_g.width;
+            tyb = t1->frame_g.y + t1->frame_g.height;
             if (! (txr >= PageLeft && txl <= PageRight
                 && tyb >= PageTop && tyt <= PageBottom)
                 && !IS_VIEWPORT_MOVED(t1)
@@ -697,9 +698,9 @@ void MoveViewport(int newx, int newy, Bool grab)
 					    t1->icon_p_height);
 			  }
 		      }
-		    SetupFrame (t1, t1->frame_x+ deltax,
-				t1->frame_y + deltay, t1->frame_width,
-				t1->frame_height,FALSE, False);
+		    SetupFrame (t1, t1->frame_g.x+ deltax,
+				t1->frame_g.y + deltay, t1->frame_g.width,
+				t1->frame_g.height,FALSE, False);
 		  }
 	      }
             /*  Bump to next win...    */
@@ -841,6 +842,11 @@ void changeDesks(int desk)
   else if ((FocusWin) && (!HAS_NEVER_FOCUS(FocusWin)))
 #endif
     SetFocus(Scr.NoFocusWin,NULL,1);
+
+#ifdef GNOME
+  GNOME_SetCurrentDesk();
+  GNOME_SetDeskCount();
+#endif
 }
 
 
@@ -879,6 +885,10 @@ void do_move_window_to_desk(FvwmWindow *tmp_win, int desk)
 	tmp_win->Desk = desk;
       BroadcastConfig(M_CONFIGURE_WINDOW,tmp_win);
     }
+#ifdef GNOME
+  GNOME_SetDeskCount();
+  GNOME_SetDesk(tmp_win);
+#endif
 }
 
 /* function with parsing of command line */
