@@ -41,6 +41,9 @@
 #include "parse.h"
 #include "screen.h"
 
+extern FvwmWindow *Tmp_win;
+extern FvwmWindow *ButtonWindow;
+
 #define SHOW_GEOMETRY (1<<0)
 #define SHOW_ALLDESKS (1<<1)
 #define SHOW_NORMAL   (1<<2)
@@ -72,7 +75,8 @@ void do_windowList(XEvent *eventp,Window w,FvwmWindow *tmp_win,
 		unsigned long context, char *action,int *Module)
 {
   MenuRoot *mr;
-  MenuItem *miExecuteAction;
+  MenuParameters mp;
+  char* ret_action = NULL;
   FvwmWindow *t;
   FvwmWindow **windowList;
   int numWindows;
@@ -97,8 +101,9 @@ void do_windowList(XEvent *eventp,Window w,FvwmWindow *tmp_win,
   MenuOptions mops;
   int low_layer = 0;  /* show all layers by default */
   int high_layer = INT_MAX;
+  int tc;
 
-  mops.flags.allflags = 0;
+  memset(&(mops.flags), 0, sizeof(mops.flags));
   if (action && *action)
   {
     /* parse postitioning args */
@@ -310,8 +315,8 @@ void do_windowList(XEvent *eventp,Window w,FvwmWindow *tmp_win,
           sprintf(loc,"%d(%d):",t->Desk, t->layer);
           strcat(tname,loc);
 
-          dheight = t->frame_height - t->title_height - 2*t->boundary_width;
-          dwidth = t->frame_width - 2*t->boundary_width;
+          dheight = t->frame_g.height - t->title_g.height -2*t->boundary_width;
+          dwidth = t->frame_g.width - 2*t->boundary_width;
 
           dwidth -= t->hints.base_width;
           dheight -= t->hints.base_height;
@@ -323,15 +328,15 @@ void do_windowList(XEvent *eventp,Window w,FvwmWindow *tmp_win,
           strcat(tname, loc);
           sprintf(loc,"x%d",dheight);
           strcat(tname, loc);
-          if(t->frame_x >=0)
-            sprintf(loc,"+%d",t->frame_x);
+          if(t->frame_g.x >=0)
+            sprintf(loc,"+%d",t->frame_g.x);
           else
-            sprintf(loc,"%d",t->frame_x);
+            sprintf(loc,"%d",t->frame_g.x);
           strcat(tname, loc);
-          if(t->frame_y >=0)
-            sprintf(loc,"+%d",t->frame_y);
+          if(t->frame_g.y >=0)
+            sprintf(loc,"+%d",t->frame_g.y);
           else
-            sprintf(loc,"%d",t->frame_y);
+            sprintf(loc,"%d",t->frame_g.y);
           strcat(tname, loc);
 
           if (IS_STICKY(t))
@@ -379,8 +384,25 @@ void do_windowList(XEvent *eventp,Window w,FvwmWindow *tmp_win,
     teventp = (XEvent *)1;
   else
     teventp = eventp;
-  menu_retval = do_menu(mr, NULL, &miExecuteAction, 0, TRUE, teventp, &mops);
-  DestroyMenu(mr);
+
+  mp.menu = mr;
+  mp.menu_prior = NULL;
+  t = Tmp_win;
+  mp.pTmp_win = &t;
+  mp.button_window = ButtonWindow;
+  tc = context;
+  mp.pcontext = &tc;
+  mp.flags.is_menu_from_frame_or_window_or_titlebar = FALSE;
+  mp.flags.is_sticky = TRUE;
+  mp.eventp = teventp;
+  mp.cmenuDeep = 0;
+  mp.pops = &mops;
+  mp.ret_paction = &ret_action;
+
+  menu_retval = do_menu(&mp);
+  if (ret_action)
+    free(ret_action);
+  DestroyMenu(mr, False);
   if (menu_retval == MENU_DOUBLE_CLICKED && default_action && *default_action)
     ExecuteFunction(default_action,tmp_win,eventp,context,*Module,
 		    EXPAND_COMMAND);
