@@ -225,6 +225,22 @@ Binding *AddBinding(Display *dpy, Binding **pblist, BindingType type,
       if (type == MOUSE_BINDING ||
 	  (tkeysym = XKeycodeToKeysym(dpy, i, m)) == keysym)
       {
+        /* If the modifier (m) doesn't change the keys value,
+           (for example, num-lock on a letter),
+           don't add the key twice, check against the one just added...
+        */
+        if (*pblist != 0 &&
+            (*pblist)->Button_Key == i &&
+            (*pblist)->type == type &&
+            (*pblist)->Context == contexts &&
+            (*pblist)->Modifier == modifiers &&
+            ((type == KEY_BINDING && strcmp((*pblist)->key_name,key_name) == 0)
+             ||
+             (type == MOUSE_BINDING && (*pblist)->key_name == NULL)) &&
+            (*pblist)->Action == action &&
+            (*pblist)->Action2 == action2) {
+          continue;
+        }
 	temp = *pblist;
 	(*pblist) = (Binding *)safemalloc(sizeof(Binding));
 	(*pblist)->type = type;
@@ -307,12 +323,16 @@ void GrabWindowKey(Display *dpy, Window w, Binding *binding,
 	 dead_modifiers != 0)
 	{
 	  register unsigned int mods;
-	  register unsigned int max = dead_modifiers | binding->Modifier;
+	  register unsigned int max = dead_modifiers;
+	  register unsigned int living_modifiers = ~dead_modifiers;
 
 	  /* handle all bindings for the dead modifiers */
 	  for (mods = 1; mods <= max; mods++)
 	    {
-	      if ((mods & ~dead_modifiers) != binding->Modifier)
+	      /* Since mods starts with 1 we don't need to test if mods
+	       * contains a dead modifier. Otherwise both, dead and living
+	       * modifiers would be zero ==> mods == 0 */
+	      if (mods & living_modifiers)
 		continue;
 	      if (fGrab)
 		XGrabKey(dpy, binding->Button_Key, mods|binding->Modifier, w,
@@ -367,12 +387,16 @@ void GrabWindowButton(Display *dpy, Window w, Binding *binding,
 	     dead_modifiers != 0)
 	    {
 	      register unsigned int mods;
-	      register unsigned int max = dead_modifiers | binding->Modifier;
+	      register unsigned int max = dead_modifiers;
+	      register unsigned int living_modifiers = ~dead_modifiers;
 
 	      /* handle all bindings for the dead modifiers */
 	      for (mods = 1; mods <= max; mods++)
 		{
-		  if ((mods & ~dead_modifiers) != binding->Modifier)
+		  /* Since mods starts with 1 we don't need to test if mods
+		   * contains a dead modifier. Otherwise both, dead and living
+		   * modifiers would be zero ==> mods == 0 */
+		  if (mods & living_modifiers)
 		    continue;
 		  if (fGrab)
 		    XGrabButton(dpy, button, mods|binding->Modifier, w, True,
