@@ -89,78 +89,13 @@ int usleep( unsigned long usec );
 
 
 
-/***********************************************************************
- * Routines for dealing with strings
- ***********************************************************************/
+#include "Strings.h"
+#include "envvar.h"
+#include "Parse.h"
 
-char *CatString3(char *a, char *b, char *c);
-void CopyString(char **dest, char *source);
-char *stripcpy( const char *source );
-int StrEquals(char *s1,char *s2);
-
-int  envExpand(char *s, int maxstrlen);
-char *envDupExpand(const char *s, int extra);
 
 int matchWildcards(char *pattern, char *string);
 
-/***********************************************************************
- * Stuff for consistent parsing
- ***********************************************************************/
-#define EatWS(s) do { while ((s) && (isspace(*(s)) || *(s) == ',')) (s)++; } while (0)
-#define IsQuote(c) ((c) == '"' || (c) == '\'' || (c) =='`')
-#define IsBlockStart(c) ((c) == '[' || (c) == '{' || (c) == '(')
-#define IsBlockEnd(c,cs) (((c) == ']' && (cs) == '[') || ((c) == '}' && (cs) == '{') || ((c) == ')' && (cs) == '('))
-#define MAX_TOKEN_LENGTH 255
-
-char *SkipQuote(char *s, const char *qlong, const char *qstart,
-		const char *qend);
-char *GetQuotedString(char *sin, char **sout, const char *delims,
-		      const char *qlong, const char *qstart, const char *qend);
-char *DoPeekToken(char *indata, char **token, char *spaces, char *delims,
-		  char *out_delim);
-char *PeekToken(char *indata, char **token);
-int MatchToken(char *pstr,char *tok);
-void NukeToken(char **pstr);
-
-/* old style parse routine: */
-char *DoGetNextToken(char *indata,char **token, char *spaces, char *delims,
-		     char *out_delim);
-char *GetNextToken(char *indata,char **token);
-char *GetNextSimpleOption(char *indata, char **option);
-char *GetNextFullOption(char *indata, char **option);
-char *SkipNTokens(char *indata, unsigned int n);
-char *GetModuleResource(char *indata, char **resource, char *module_name);
-int GetSuffixedIntegerArguments(char *action, char **ret_action, int retvals[],
-				int num, char *suffixlist,
-				int ret_suffixnum[]);
-int GetIntegerArguments(char *action, char**ret_action, int retvals[],int num);
-int GetTokenIndex(char *token, char *list[], int len, char **next);
-char *GetNextTokenIndex(char *action, char *list[], int len, int *index);
-int GetRectangleArguments(char *action, int *width, int *height);
-int GetOnePercentArgument(char *action, int *value, int *unit_io);
-int GetTwoPercentArguments(char *action, int *val1, int *val2, int *val1_unit,
-			   int *val2_unit);
-int ParseToggleArgument(char *action, char **ret_action, int default_ret,
-			char no_toggle);
-
-/*
-   function:            FindToken
-   description:         find the entry of type 'struct_entry'
-                        holding 'key' in 'table'
-   returns:             pointer to the matching entry
-                        NULL if not found
-
-   table must be sorted in ascending order for FindToken.
-*/
-
-#define FindToken(key,table,struct_entry)                               \
-        (struct_entry *) bsearch(key,                                   \
-                                 (char *)(table),                       \
-                                 sizeof(table) / sizeof(struct_entry),  \
-                                 sizeof(struct_entry),                  \
-                                 XCmpToken)
-
-extern int XCmpToken(); /* const char *s, const char **t); */
 
 
 /***********************************************************************
@@ -211,14 +146,6 @@ char* searchPath( const char* path, const char* filename,
 		  const char* suffix, int type );
 
 
-/***********************************************************************
- * Stuff for modules to communicate with fvwm
- ***********************************************************************/
-int ReadFvwmPacket(int fd, unsigned long *header, unsigned long **body);
-void SendText(int *fd,char *message,unsigned long window);
-#define SendInfo SendText
-void GetConfigLine(int *fd, char **tline);
-void SetMessageMask(int *fd, unsigned long mask);
 
 /***********************************************************************
  * Stuff for dealing w/ bitmaps & pixmaps:
@@ -252,10 +179,69 @@ Bool GetResourceString(XrmDatabase db, const char *resource,
 		       const char *prefix, char **val);
 
 
-/* things in Graphics.c */
+/***********************************************************************
+ * Stuff for Graphics.c and ModGraph.c
+ ***********************************************************************/
+
+/* stuff to enable modules to use fvwm visual/colormap/GCs */
+#define DEFGRAPHSTR "Default_Graphics"
+#define DEFGRAPHLEN 16 /* length of above string */
+#define DEFGRAPHNUM 9 /* number of items sent */
+
+typedef struct _Background {
+  union {
+    unsigned long word;
+    struct {
+      Bool is_pixmap : 1;
+      Bool stretch_h : 1;
+      Bool stretch_v : 1;
+      unsigned int w : 12;
+      unsigned int h : 12;
+    } bits;
+  } type;
+  Pixmap pixmap;
+} Background;
+
+typedef struct GraphicsThing {
+  Bool create_drawGC : 1;
+  Bool create_foreGC : 1;
+  Bool create_reliefGC : 1;
+  Bool create_shadowGC : 1;
+  Bool initialised : 1;
+  Bool useFvwmLook : 1;
+  Visual *viz;
+  unsigned int depth;
+  Colormap cmap;
+  GC drawGC;
+  Background *bg;
+  GC foreGC;
+  GC reliefGC;
+  GC shadowGC;
+  XFontStruct *font;
+} Graphics;
+
+Graphics *CreateGraphics(void);
+void InitGraphics(Display *dpy, Graphics *graphics);
+Bool ParseGraphics(Display *dpy, char *line, Graphics *graphics);
+
 void RelieveRectangle(Display *dpy, Window win, int x,int y,int w,int h,
 		      GC ReliefGC, GC ShadowGC, int line_width);
 
+void SetWindowBackground(Display *dpy, Window win, int width, int height,
+			 Background *background, unsigned int depth, GC gc);
+
+Pixmap CreateStretchXPixmap(Display *dpy, Pixmap src, unsigned int src_width,
+			    unsigned int src_height, unsigned int src_depth,
+			    unsigned int dest_width, GC gc);
+
+Pixmap CreateStretchYPixmap(Display *dpy, Pixmap src, unsigned int src_width,
+			    unsigned int src_height, unsigned int src_depth,
+			    unsigned int dest_height, GC gc);
+
+Pixmap CreateStretchPixmap(Display *dpy, Pixmap src, unsigned int src_width,
+			    unsigned int src_height, unsigned int src_depth,
+			    unsigned int dest_width, unsigned int dest_height,
+			    GC gc);
 
 /***********************************************************************
  * Key and mouse bindings
