@@ -23,6 +23,7 @@
 
 #include <ctype.h>
 
+#include <X11/Xlib.h>
 #include <X11/StringDefs.h>
 #include <X11/Intrinsic.h>
 #include <X11/Shell.h>
@@ -32,6 +33,7 @@
 
 #include <libs/fvwmlib.h>
 #include <libs/Picture.h>
+#include <libs/Module.h>
 
 
 #include "icons/fvwm2_big.xpm"
@@ -76,6 +78,7 @@ Pixel back_pix, fore_pix;
 GC NormalGC,FGC;
 static Atom wm_del_win;
 Colormap colormap;
+Bool no_wm = False;
 
 #define MW_EVENTS   (ExposureMask | ButtonReleaseMask)
 
@@ -90,11 +93,10 @@ int main(int argc, char **argv)
   int retval = 0;
   XEvent Event;
   fd_set in_fdset;
-  int fd_width ;
+  fd_set_size_t fd_width = GetFdWidth();
   struct timeval value;
   int fd[2];
-
-  fd_width = GetFdWidth();
+  XSetWindowAttributes attr;
 
   /* Save our program  name - for error messages */
   string = strrchr (argv[0], '/');
@@ -182,7 +184,10 @@ int main(int argc, char **argv)
 
 
   /* Set assorted info for the window */
-  XSetTransientForHint(dpy,win,Root);
+  if (no_wm) {
+    attr.override_redirect = True;
+    XChangeWindowAttributes(dpy, win, CWOverrideRedirect, &attr);
+  }
   wm_del_win = XInternAtom(dpy,"WM_DELETE_WINDOW",False);
   XSetWMProtocols(dpy,win,&wm_del_win,1);
 
@@ -210,7 +215,7 @@ int main(int argc, char **argv)
 
     if(!XPending(dpy))
 
-      retval=select(fd_width,SELECT_TYPE_ARG234 &in_fdset, 0, 0, &value);
+      retval=select(fd_width,SELECT_FD_SET_CAST &in_fdset, 0, 0, &value);
 
     if (retval==0)
     {
@@ -321,6 +326,13 @@ static void parseOptions (int fd[2])
 	}
         continue;
       }
+      if (strncasecmp (tline,
+			 CatString3 ("*", myName, "NoDecor"),
+			 clength + 8) ==0)
+      {
+        no_wm = True;
+        continue;
+      }
       if (strncasecmp (tline,
 			 CatString3 ("*", myName, "Timeout"),
 			 clength + 8) ==0)
