@@ -58,7 +58,7 @@
 #include <X11/extensions/shape.h>
 #endif
 
-#include "fvwm/module.h"
+#include "libs/Module.h"
 #include "libs/fvwmlib.h"
 #include "FvwmButtons.h"
 #include "misc.h" /* ConstrainSize() */
@@ -120,13 +120,15 @@ void Slide(panel_info *, button_info *);
 
 Display *Dpy;
 Window Root;
+Graphics *G;
+GC trans_gc = NULL;
 Window MyWindow;
 char *MyName;
 XFontStruct *font;
 int screen;
-int d_depth;
 
-int x_fd,fd_width;
+int x_fd;
+fd_set_size_t fd_width;
 
 char *config_file = NULL;
 
@@ -139,6 +141,8 @@ char *imagePath = NULL;
 
 Pixel hilite_pix, back_pix, shadow_pix, fore_pix;
 GC  NormalGC;
+/* needed for relief drawing only */
+GC  ShadowGC;
 int Width,Height;
 
 int x= -30000,y= -30000,w= -1,h= -1,gravity = NorthWestGravity;
@@ -405,9 +409,6 @@ void SetTransparentBackground(button_info *ub,int w,int h)
 {
   Pixmap pmap_mask;
   button_info *b;
-  GC gc;
-  XGCValues gvals;
-  unsigned long gcm=0;
   Window root_return;
   int x_return, y_return;
   unsigned int width_return, height_return;
@@ -417,10 +418,17 @@ void SetTransparentBackground(button_info *ub,int w,int h)
   XFontStruct *font;
 
   pmap_mask = XCreatePixmap(Dpy,MyWindow,w,h,1);
-  gc = XCreateGC(Dpy,pmap_mask,(unsigned long)0,&gvals);
-  XSetForeground(Dpy,gc,0);
-  XFillRectangle(Dpy,pmap_mask,gc,0,0,w,h);
-  XSetForeground(Dpy,gc,1);
+
+  if (trans_gc == NULL) {
+    XGCValues gcv;
+
+    /* create a GC for doing transparency */
+    trans_gc = XCreateGC(Dpy,pmap_mask,(unsigned long)0,&gcv);
+  }
+
+  XSetForeground(Dpy,trans_gc,0);
+  XFillRectangle(Dpy,pmap_mask,trans_gc,0,0,w,h);
+  XSetForeground(Dpy,trans_gc,1);
 
   /*
    * if button has an icon, draw a rect with the same size as the icon
@@ -438,17 +446,18 @@ void SetTransparentBackground(button_info *ub,int w,int h)
 		       &border_width_return,&depth_return);
 
 	  number=buttonNum(b);
-	  if (b->icon->mask == None)
-	    XFillRectangle(Dpy,pmap_mask,gc,x_return, y_return,
+	  if (b->icon->mask == None) {
+	    XFillRectangle(Dpy,pmap_mask,trans_gc,x_return, y_return,
 			   b->icon->width,b->icon->height);
-	  else
-	    XCopyArea(Dpy,b->icon->mask,pmap_mask,gc,0,0,
+	  } else {
+	    XCopyArea(Dpy,b->icon->mask,pmap_mask,trans_gc,0,0,
 		      b->icon->width,b->icon->height,x_return,y_return);
+	  }
 	}
       else
 	{
 	  number=buttonNum(b);
-	  XFillRectangle(Dpy,pmap_mask,gc,buttonXPos(b,number),
+	  XFillRectangle(Dpy,pmap_mask,trans_gc,buttonXPos(b,number),
 			 buttonYPos(b,number),
 			 buttonWidth(b),buttonHeight(b));
 	}
@@ -457,14 +466,10 @@ void SetTransparentBackground(button_info *ub,int w,int h)
       font=buttonFont(b);
       if(b->flags&b_Title && font)
 	{
-	  gcm = GCForeground | GCFont;
-	  gvals.foreground=1;
-	  gvals.font = font->fid;
-	  XChangeGC(Dpy,gc,gcm,&gvals);
-	  DrawTitle(b,pmap_mask,gc);
+	  XSetFont(Dpy,trans_gc,font->fid);
+	  DrawTitle(b,pmap_mask,trans_gc);
 	}
     }
-  XFreeGC(Dpy,gc);
   XShapeCombineMask(Dpy,MyWindow,ShapeBounding,0,0,pmap_mask,ShapeSet);
 }
 #endif
@@ -494,7 +499,6 @@ int myErrorHandler(Display *dpy, XErrorEvent *event)
 **/
 int main(int argc, char **argv)
 {
-  char *display_name = NULL;
   int i;
   Window root;
   int x,y,maxx,maxy,border_width,depth;
@@ -555,12 +559,18 @@ int main(int argc, char **argv)
 
   fd[0]=atoi(argv[1]);
   fd[1]=atoi(argv[2]);
-  if (!(Dpy = XOpenDisplay(display_name)))
+  if (!(Dpy = XOpenDisplay(NULL)))
     {
       fprintf(stderr,"%s: Can't open display %s", MyName,
-	      XDisplayName(display_name));
+	      XDisplayName(NULL));
       exit (1);
     }
+  G = CreateGraphics();
+  G->create_foreGC = True;
+  G->create_reliefGC = True;
+  G->create_shadowGC = True;
+  InitGraphics(Dpy, G);
+
   x_fd=XConnectionNumber(Dpy);
   fd_width=GetFdWidth();
 
@@ -571,7 +581,6 @@ int main(int argc, char **argv)
       fprintf(stderr,"%s: Screen %d is not valid\n",MyName,screen);
       exit(1);
     }
-  d_depth = DefaultDepth(Dpy, screen);
 
   oldErrorHandler=XSetErrorHandler(myErrorHandler);
 
@@ -597,6 +606,7 @@ int main(int argc, char **argv)
   UberButton->swallow = 1; /* the panel is shown */
 
   ParseOptions(UberButton);
+  SavePictureCMap(Dpy, G->viz, G->cmap, G->depth); /* store cmap */
 
   for (CurrentPanel = MainPanel, LastPanel = NULL;
        CurrentPanel != NULL;
@@ -633,7 +643,6 @@ int main(int argc, char **argv)
 
     /* Load fonts and icons, calculate max buttonsize */
     maxx=0;maxy=0;
-    InitPictureCMap(Dpy,Root); /* store the root cmap */
     RecursiveLoadData(UberButton,&maxx,&maxy);
 
 # ifdef DEBUG_INIT
@@ -691,8 +700,9 @@ int main(int argc, char **argv)
   XMapSubwindows(Dpy,MyWindow);
   XMapWindow(Dpy,MyWindow);
 
-  SetMessageMask(fd, M_NEW_DESK | M_END_WINDOWLIST | M_MAP | M_WINDOW_NAME |
-		 M_RES_CLASS | M_CONFIG_INFO | M_END_CONFIG_INFO | M_RES_NAME);
+  SetMessageMask(fd, M_NEW_DESK | M_END_WINDOWLIST | M_MAP | M_WINDOW_NAME
+		 | M_RES_CLASS | M_CONFIG_INFO | M_END_CONFIG_INFO | M_RES_NAME
+		 | M_SENDCONFIG);
 
   /* request a window list, since this triggers a response which
    * will tell us the current desktop and paging status, needed to
@@ -1045,12 +1055,17 @@ void RedrawWindow(button_info *b)
   int button;
   XEvent dummy;
   button_info *ub;
+  XGCValues gcv;
+  unsigned long gcm=0;
+  static Bool initial_redraw = True;
+  Bool clear_buttons;
 
   if(ready<1)
     return;
 
   /* Flush expose events */
-  while (XCheckTypedWindowEvent (Dpy, MyWindow, Expose, &dummy));
+  while (XCheckTypedWindowEvent (Dpy, MyWindow, Expose, &dummy))
+    ;
 
   if(b)
     {
@@ -1058,7 +1073,24 @@ void RedrawWindow(button_info *b)
       return;
     }
 
-  button=-1;ub=UberButton;
+  /* Clean out the entire window first */
+  if (initial_redraw == False)
+  {
+    gcm = GCBackground;
+    gcv.foreground = fore_pix;
+    gcv.background = back_pix;
+    XChangeGC(Dpy,NormalGC,gcm,&gcv);
+    XFillRectangle(Dpy,MyWindow,NormalGC,0,0,Width,Height);
+    clear_buttons = False;
+  }
+  else
+  {
+    initial_redraw = False;
+    clear_buttons = True;
+  }
+
+  button=-1;
+  ub=UberButton;
   while(NextButton(&ub,&b,&button,1))
     RedrawButton(b,1);
 }
@@ -1261,6 +1293,7 @@ void CreateWindow(button_info *ub,int maxx,int maxy)
   XGCValues gcv;
   unsigned long gcm;
   XClassHint myclasshints;
+  XSetWindowAttributes xswa;
 
   x = CurrentPanel->uber->x; /* Geometry x where to put the panel */
   y = CurrentPanel->uber->y; /* Geometry y where to put the panel */
@@ -1344,24 +1377,6 @@ void CreateWindow(button_info *ub,int maxx,int maxy)
     }
   mysizehints.win_gravity = gravity;
 
-# ifdef DEBUG_INIT
-  fprintf(stderr,"colors...");
-# endif
-  if(d_depth < 2)
-    {
-      back_pix = GetColor("white");
-      fore_pix = GetColor("black");
-      hilite_pix = back_pix;
-      shadow_pix = fore_pix;
-    }
-  else
-    {
-      back_pix = GetColor(ub->c->back);
-      fore_pix = GetColor(ub->c->fore);
-      hilite_pix = GetHilite(back_pix);
-      shadow_pix = GetShadow(back_pix);
-    }
-
 # ifdef DEBUG_INIT
   if(mysizehints.flags&USPosition)
     fprintf(stderr,"create(%i,%i,%u,%u,1,%u,%u)...",
@@ -1374,11 +1389,39 @@ void CreateWindow(button_info *ub,int maxx,int maxy)
 	    (ushort)fore_pix,(ushort)back_pix);
 # endif
 
-  MyWindow = XCreateSimpleWindow(Dpy,Root,mysizehints.x,mysizehints.y,
-				 mysizehints.width,mysizehints.height,
-				 0,fore_pix,back_pix);
-  if(ub->c->flags&b_IconBack && !(ub->c->flags&b_TransBack))
-    XSetWindowBackgroundPixmap(Dpy,MyWindow,ub->c->backicon->picture);
+  xswa.colormap = G->cmap;
+  xswa.border_pixel = 0;
+  xswa.background_pixmap = None;
+  MyWindow = XCreateWindow(Dpy,Root,mysizehints.x,mysizehints.y,
+			   mysizehints.width,mysizehints.height,0,G->depth,
+			   InputOutput,G->viz,
+			   CWColormap|CWBackPixmap|CWBorderPixel,&xswa);
+
+# ifdef DEBUG_INIT
+  fprintf(stderr,"colors...");
+# endif
+  if (!(ub->c->flags&b_FvwmLook)) {
+    if(G->depth < 2) {
+      back_pix = GetColor("white");
+      fore_pix = GetColor("black");
+      hilite_pix = back_pix;
+      shadow_pix = fore_pix;
+    } else {
+      back_pix = GetColor(ub->c->back);
+      fore_pix = GetColor(ub->c->fore);
+      hilite_pix = GetHilite(back_pix);
+      shadow_pix = GetShadow(back_pix);
+    }
+    if(ub->c->flags&b_IconBack && !(ub->c->flags&b_TransBack)) {
+      XSetWindowBackgroundPixmap(Dpy,MyWindow,ub->c->backicon->picture);
+    } else {
+      XSetWindowBackground(Dpy,MyWindow,back_pix);
+    }
+  } else {
+    SetWindowBackground(Dpy,MyWindow,mysizehints.width,mysizehints.height,
+			G->bg, G->depth, G->foreGC);
+  }
+
 
 
 # ifdef DEBUG_INIT
@@ -1431,7 +1474,10 @@ void CreateWindow(button_info *ub,int maxx,int maxy)
       gcv.font = ub->c->font->fid;
       gcm |= GCFont;
     }
-  NormalGC = XCreateGC(Dpy, Root, gcm, &gcv);
+  NormalGC = XCreateGC(Dpy, MyWindow, gcm, &gcv);
+  gcv.foreground = shadow_pix;
+  gcv.background = fore_pix;
+  ShadowGC = XCreateGC(Dpy, MyWindow, gcm, &gcv);
 
   free(myclasshints.res_class);
   free(myclasshints.res_name);
@@ -1460,8 +1506,11 @@ int PleaseAllocColor(XColor *color)
 
   sprintf(buf,"x c #%04x%04x%04x",color->red,color->green,color->blue);
   xpm[1]=buf;
-  attr.valuemask=XpmCloseness;
+  attr.valuemask=XpmCloseness|XpmVisual|XpmColormap|XpmDepth;
   attr.closeness=40000; /* value used by fvwm and fvwmlib */
+  attr.visual = G->viz;
+  attr.colormap = G->cmap;
+  attr.depth = G->depth;
 
   if(XpmCreateImageFromData(Dpy,xpm,&dummy1,&dummy2,&attr)!=XpmSuccess)
     {
@@ -1492,13 +1541,11 @@ void nocolor(const char *a, const char *b)
 Pixel GetColor(char *name)
 {
   XColor color;
-  XWindowAttributes attributes;
 
-  XGetWindowAttributes(Dpy,Root,&attributes);
   color.pixel = 0;
-  if (!XParseColor (Dpy, attributes.colormap, name, &color))
+  if (!XParseColor (Dpy, G->cmap, name, &color))
     nocolor("parse",name);
-  else if(!MyAllocColor(Dpy,attributes.colormap,&color))
+  else if(!MyAllocColor(Dpy,G->cmap,&color))
     nocolor("alloc",name);
   return color.pixel;
 }
@@ -1587,7 +1634,7 @@ int My_XNextEvent(Display *Dpy, XEvent *event)
   FD_SET(x_fd,&in_fdset);
   FD_SET(fd[1],&in_fdset);
 
-  if (select(fd_width,SELECT_TYPE_ARG234 &in_fdset, 0, 0, NULL) > 0)
+  if (select(fd_width,SELECT_FD_SET_CAST &in_fdset, 0, 0, NULL) > 0)
   {
 
   if(FD_ISSET(x_fd, &in_fdset))
@@ -1679,6 +1726,19 @@ void process_message(unsigned long type,unsigned long *body)
     case M_WINDOW_NAME:
       CheckForHangon(body);
       break;
+    case M_CONFIG_INFO:
+      { /* there's only one config line of interest at this point */
+	char *line = (char *)&body[3];
+	if ((strncasecmp(line, DEFGRAPHSTR, DEFGRAPHLEN)==0)
+	    && (ParseGraphics(Dpy, line, G))
+	    && (UberButton->c->flags & b_FvwmLook))
+	  SetWindowBackground(Dpy, MyWindow, UberButton->c->ButtonWidth
+			      * UberButton->c->num_columns,
+			      UberButton->c->ButtonHeight
+			      * UberButton->c->num_rows, G->bg, G->depth,
+			      G->foreGC);
+      }
+      break;
     default:
       break;
     }
