@@ -82,7 +82,7 @@
 #define XK_MISCELLANY
 #include <X11/keysymdef.h>
 
-#include "fvwm/module.h"                /* for headersize, etc. */
+#include "libs/Module.h"                /* for headersize, etc. */
 #include "libs/fvwmlib.h"
 
 #include <libs/Picture.h>               /* for InitPictureCMap */
@@ -342,7 +342,7 @@ static void ct_Line(char *cp) {
   cur_line->next=calloc(sizeof(struct _line),1); /* malloc new line */
   if (cur_line->next == 0) {
     fprintf(stderr, "%s: Malloc for line, (%d bytes) failed. exiting.\n",
-            MyName+1, sizeof(struct _line));
+            MyName+1, (int)sizeof(struct _line));
     exit (1);
   }
   cur_line = cur_line->next;            /* new current line */
@@ -374,10 +374,10 @@ static void ct_Message(char *cp) {
   item->header.name = "FvwmMessage";    /* No purpose to this? dje */
   item->text.value = malloc(80);        /* point at last error recvd */
   item->text.n = 80;
-  memset(item->text.value,'M',80);      /* for font width calc */
-  item->header.size_x = XTextWidth(item->header.dt_ptr->dt_font_struct,
+  strcpy(item->text.value,"A mix of chars. MM20"); /* 20 mixed width chars */
+  item->header.size_x = (XTextWidth(item->header.dt_ptr->dt_font_struct,
                                    item->text.value,
-                                   item->text.n) + 2 * TEXT_SPC;
+                                   item->text.n/4) * 4) + 2 * TEXT_SPC;
   item->header.size_y = FontHeight(item->header.dt_ptr->dt_font_struct)
     + CF.padVText;
 
@@ -469,7 +469,7 @@ static void AssignDrawTable(Item *adt_item) {
   if (new_dt == 0) {                /* malloc failed? */
     fprintf(stderr,
             "%s: Malloc for DrawTable, (%d bytes) failed. exiting.\n",
-            MyName+1, sizeof(struct _drawtable));
+            MyName+1, (int)sizeof(struct _drawtable));
     exit (1);                           /* give up */
   }
   new_dt->dt_next = 0;                  /* new end of list */
@@ -502,7 +502,7 @@ static void AddItem() {
   item = calloc(sizeof(Item),1);        /* get a new item */
   if (item == 0) {                      /* if out of mem */
     fprintf(stderr, "%s: Malloc for item, (%d bytes) failed. exiting.\n",
-            MyName+1, sizeof(Item));
+            MyName+1, (int)sizeof(Item));
     exit (1);                           /* give up */
   }
   if (save_item == 0) {                 /* if first item */
@@ -788,7 +788,7 @@ static void ct_Command(char *cp) {
     if (cur_button->button.commands == 0) {
       fprintf(stderr,"%s: realloc button array size %d failed\n",
               MyName+1, cur_button->button.button_array_size *
-              sizeof(char *));
+              (int)sizeof(char *));
       exit (1);
     }
   }
@@ -1449,7 +1449,7 @@ static void MainLoop ()
     FD_SET(fd_x, &fds);
 
     XFlush(dpy);
-    if (select(32, &fds, NULL, NULL, NULL) > 0) {
+    if (select(32, SELECT_FD_SET_CAST &fds, NULL, NULL, NULL) > 0) {
       if (FD_ISSET(Channel[1], &fds))
 	ReadFvwm();
       if (FD_ISSET(fd_x, &fds))
