@@ -37,7 +37,7 @@
 #include <unistd.h>
 #include <ctype.h>
 #include <stdlib.h>
-#include "fvwm/module.h"
+#include "libs/Module.h"
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -64,7 +64,8 @@ char *MyName;
 XFontStruct *font;
 
 Display *dpy;			/* which display are we talking to */
-int x_fd,fd_width;
+int x_fd;
+fd_set_size_t fd_width;
 
 Window Root;
 int screen;
@@ -161,7 +162,7 @@ int ready = 0;
 unsigned long local_flags = 0;
 int sortby = UNSORT;
 
-int save_color_limit;                   /* color limit from config */
+int save_color_limit = 0;                   /* color limit from config */
 
 /************************************************************************
   Main
@@ -1483,7 +1484,7 @@ void ParseOptions(void)
 					      ""),Clength+1)==0)
 	  parseicon(&tline[Clength + 1]);
 	else if (strncasecmp(tline,"ImagePath",9)==0)
-	  CopyString(&imagePath,&tline[8]);
+	  CopyString(&imagePath,&tline[9]);
 	else if (strncasecmp(tline,"ClickTime",9)==0)
 	  ClickTime = atoi(&tline[9]);
 	else if (strncasecmp(tline,"ColorLimit",10)==0) {
@@ -1744,7 +1745,7 @@ int My_XNextEvent(Display *dpy, XEvent *event)
   FD_SET(x_fd,&in_fdset);
   FD_SET(fd[1],&in_fdset);
 
-  select(fd_width,SELECT_TYPE_ARG234 &in_fdset, 0, 0, NULL);
+  select(fd_width,SELECT_FD_SET_CAST &in_fdset, 0, 0, NULL);
 
   if(FD_ISSET(x_fd, &in_fdset))
     {
