@@ -34,7 +34,7 @@
 #include <X11/Xatom.h>
 #include <X11/Intrinsic.h>
 
-#include "fvwm/module.h"
+#include "libs/Module.h"
 
 #include "FvwmM4.h"
 #include "libs/fvwmlib.h"
