@@ -33,7 +33,7 @@
 #include <X11/Xatom.h>
 #include <X11/Intrinsic.h>
 
-#include "fvwm/module.h"
+#include "libs/Module.h"
 #include "libs/fvwmlib.h"
 
 #include "fvwm/fvwm.h"
@@ -57,7 +57,8 @@ PagerWindow *Start = NULL;
 PagerWindow *FocusWin = NULL;
 
 Display *dpy;			/* which display are we talking to */
-int x_fd,fd_width;
+int x_fd;
+fd_set_size_t fd_width;
 
 char *PagerFore = NULL;
 char *PagerBack = NULL;
@@ -78,6 +79,9 @@ Picture *PixmapBack = NULL;
 
 char *ImagePath = NULL;
 
+#define DEFAULT_MOVE_THRESHOLD 3
+int MoveThreshold = DEFAULT_MOVE_THRESHOLD;
+
 int ShowBalloons = 0, ShowPagerBalloons = 0, ShowIconBalloons = 0;
 char *BalloonTypeString = NULL;
 char *BalloonBack = NULL;
@@ -109,6 +113,7 @@ Pixel win_hi_fore_pix = -1;
 char fAlwaysCurrentDesk = 0;
 PagerStringList string_list = { NULL, 0, NULL, NULL, NULL };
 Bool is_transient = False;
+Bool error_occured = False;
 
 static volatile sig_atomic_t isTerminated = False;
 
@@ -370,6 +375,26 @@ void Loop(int *fd)
     {
       if(My_XNextEvent(dpy,&Event))
 	DispatchEvent(&Event);
+      if (error_occured)
+      {
+	Window root;
+	unsigned border_width, depth;
+	int x,y;
+
+	if(XGetGeometry(dpy,Scr.Pager_w,&root,&x,&y,
+			(unsigned *)&window_w,(unsigned *)&window_h,
+			&border_width,&depth)==0)
+	  {
+	    if (is_transient)
+	      {
+		XUngrabPointer(dpy,CurrentTime);
+		MyXUngrabServer(dpy);
+		XSync(dpy,0);
+	      }
+	    exit(0);
+	  }
+	error_occured = False;
+      }
     }
 }
 
@@ -498,7 +523,7 @@ void list_add(unsigned long *body)
   (*prev)->width = cfgpacket->frame_width;
   (*prev)->height = cfgpacket->frame_height;
   (*prev)->desk = cfgpacket->desk;
-  memcpy(&((*prev)->flags), &(cfgpacket->flags), sizeof(cfgpacket->flags)); 
+  memcpy(&((*prev)->flags), &(cfgpacket->flags), sizeof(cfgpacket->flags));
   (*prev)->title_height = cfgpacket->title_height;
   (*prev)->border_width = cfgpacket->border_width;
   (*prev)->icon_w = cfgpacket->icon_w;
@@ -549,7 +574,7 @@ void list_configure(unsigned long *body)
       t->frame_height = cfgpacket->frame_height;
       t->title_height = cfgpacket->title_height;
       t->border_width = cfgpacket->border_width;
-      memcpy(&(t->flags), &(cfgpacket->flags), sizeof(cfgpacket->flags)); 
+      memcpy(&(t->flags), &(cfgpacket->flags), sizeof(cfgpacket->flags));
       t->icon_w = cfgpacket->icon_w;
       t->icon_pixmap_w = cfgpacket->icon_pixmap_w;
 
@@ -1153,7 +1178,7 @@ int My_XNextEvent(Display *dpy, XEvent *event)
   FD_SET(x_fd,&in_fdset);
   FD_SET(fd[1],&in_fdset);
 
-  if (select(fd_width,SELECT_TYPE_ARG234 &in_fdset, 0, 0, NULL) > 0)
+  if (select(fd_width,SELECT_FD_SET_CAST &in_fdset, 0, 0, NULL) > 0)
   {
   if(FD_ISSET(x_fd, &in_fdset))
     {
@@ -1221,24 +1246,42 @@ void ParseOptions(void)
       char *arg1;
       char *arg2;
       char *tline2;
+      char *token;
+      char *next;
+      Bool MoveThresholdSetForModule = False;
 
       resource_string = arg1 = arg2 = NULL;
 
-      if (MatchToken(tline, "ImagePath")) {
-	  if (ImagePath != NULL) {
+      token = PeekToken(tline, &next);
+      if (StrEquals(token, "ImagePath"))
+      {
+	  if (ImagePath != NULL)
+	  {
 	      free(ImagePath);
 	      ImagePath = NULL;
 	  }
-
-	  tline = SkipNTokens( tline, 1 );
-
-	  CopyString(&ImagePath, tline);
+	  GetNextToken(next, &ImagePath);
 
 #ifdef DEBUG
 	  fprintf(stderr, "[ParseOptions]: ImagePath = %s\n", ImagePath);
 #endif
 	  continue;
       }
+      else if (StrEquals(token, "MoveThreshold"))
+      {
+	if (!MoveThresholdSetForModule)
+	{
+	  int val;
+	  if (GetIntegerArguments(next, NULL, &val, 1) > 0)
+	  {
+	    if (val >= 0)
+	      MoveThreshold = val;
+	    else
+	      MoveThreshold = DEFAULT_MOVE_THRESHOLD;
+	  }
+	}
+	continue;
+      }
 
       tline2 = GetModuleResource(tline, &resource, MyName);
       if (!resource)
@@ -1613,13 +1656,21 @@ void ParseOptions(void)
 	      GetNextToken(tline2, &WindowHiBack);
 	    }
 	}
-       else if (StrEquals(resource,"WindowLabelFormat"))
-         {
-           if (WindowLabelFormat)
-             free(WindowLabelFormat);
-           CopyString(&WindowLabelFormat,arg1);
-         }
-
+      else if (StrEquals(resource,"WindowLabelFormat"))
+	{
+	  if (WindowLabelFormat)
+	    free(WindowLabelFormat);
+	  CopyString(&WindowLabelFormat,arg1);
+	}
+      else if (StrEquals(resource, "MoveThreshold"))
+	{
+	  int val;
+	  if (GetIntegerArguments(next, NULL, &val, 1) > 0 && val >= 0)
+	    {
+	      MoveThreshold = val;
+	      MoveThresholdSetForModule = True;
+	    }
+	}
       /* ... and get Balloon config options ...
          -- ric@giccs.georgetown.edu */
       else if (StrEquals(resource, "Balloons"))
