@@ -6,7 +6,7 @@
  *
  * Minor hack by TKP to enable autohide to work with pages (leaves 2 pixels
  * visible so that the 1 pixel panframes don't get in the way)
- * 
+ *
  * Merciless hack by RBW for the Great Style Flag Rewrite - 04/20/1999.
  * This module slightly abused the old flags word, inventing new meanings for
  * certain bits. Therefore, I'm leaving all that infrastructure alone, only
@@ -58,7 +58,7 @@
 
 #include <stdlib.h>
 
-#include "fvwm/module.h"
+#include "libs/Module.h"
 #include "libs/fvwmlib.h"  /* for pixmaps routines */
 
 
@@ -86,7 +86,7 @@ GC checkered;
 
 /* File type information */
 FILE  *console;
-int   fd_width;
+fd_set_size_t  fd_width;
 int   Fvwm_fd[2];
 int   x_fd;
 
@@ -156,6 +156,9 @@ Colormap PictureCMap;
 
 char *ImagePath   = NULL;
 
+static Bool tip_window_alarm = False;
+static Bool hide_taskbar_alarm = False;
+
 static void ParseConfig( void );
 static void ParseConfigLine(char *tline);
 
@@ -200,12 +203,11 @@ int main(int argc, char **argv)
   signal (SIGALRM, Alarm);
 
   SetMessageMask(Fvwm_fd,M_ADD_WINDOW | M_CONFIGURE_WINDOW | M_DESTROY_WINDOW |
-		 M_WINDOW_NAME | M_ICON_NAME | M_RES_NAME | M_DEICONIFY | M_ICONIFY |
-		 M_END_WINDOWLIST | M_FOCUS_CHANGE |
-		M_CONFIG_INFO | M_END_CONFIG_INFO
+		 M_WINDOW_NAME | M_ICON_NAME | M_RES_NAME | M_DEICONIFY |
+		 M_ICONIFY | M_END_WINDOWLIST | M_FOCUS_CHANGE |
+		 M_CONFIG_INFO | M_END_CONFIG_INFO
 #ifdef FVWM95
-		| M_FUNCTION_END |
-		M_SCROLLREGION
+		 | M_FUNCTION_END | M_SCROLLREGION
 #endif
 #ifdef MINI_ICONS
 		 | M_MINI_ICON
@@ -256,24 +258,8 @@ void EndLessLoop()
     XPending(dpy);
     tv.tv_sec  = 0;
     tv.tv_usec = 0;
-#ifdef __hpux
-    if (!select(fd_width, (int *)&readset, NULL, NULL, &tv)) {
-      while(1) {
-        FD_ZERO(&readset);
-        FD_SET(Fvwm_fd[1], &readset);
-        FD_SET(x_fd, &readset);
-        XPending(dpy);
 
-        tv.tv_sec  = UpdateInterval;
-        tv.tv_usec = 0;
-        if (select(fd_width, (int *)&readset, NULL, NULL, &tv) <= 0)
-          DrawGoodies();
-        else
-          break;
-      }
-    }
-#else
-    if (!select(fd_width, &readset, NULL, NULL, &tv)) {
+    if (!select(fd_width, SELECT_FD_SET_CAST &readset, NULL, NULL, &tv)) {
       while(1) {
         FD_ZERO(&readset);
         FD_SET(Fvwm_fd[1], &readset);
@@ -283,13 +269,12 @@ void EndLessLoop()
         tv.tv_sec  = UpdateInterval;
         tv.tv_usec = 0;
 
-        if (select(fd_width, &readset, NULL, NULL, &tv) <= 0)
+        if (select(fd_width, SELECT_FD_SET_CAST &readset, NULL, NULL, &tv) <= 0)
           DrawGoodies();
         else
           break;
       }
     }
-#endif
 
     if (FD_ISSET(x_fd, &readset))
       LoopOnEvents();
@@ -352,7 +337,8 @@ void ProcessMessage(unsigned long type,unsigned long *body)
        Window redraw and rewarping gets handled by XEvent
        ConfigureNotify code. */
     cfgpacket = (ConfigWinPacket *) body;
-    if (!ShowTransients && (IS_TRANSIENT(cfgpacket))) break;
+    if (!ShowTransients && (IS_TRANSIENT(cfgpacket)))
+      break;
     if (cfgpacket->w == win) {
       if (win_border != (int)cfgpacket->border_width) {
 	win_x = win_border = (int)cfgpacket->border_width;
@@ -364,12 +350,12 @@ void ProcessMessage(unsigned long type,unsigned long *body)
 
         XMoveResizeWindow(dpy, win, win_x, win_y,
                           ScreenWidth-(win_border<<1), win_height);
-
       }
       break;
     }
 
-    if (FindItem(&windows, cfgpacket->w) != -1) break;
+    if (FindItem(&windows, cfgpacket->w) != -1)
+      break;
     if (!(DO_SKIP_WINDOW_LIST(cfgpacket)) || !UseSkipList) {
       AddItem(&windows,
         cfgpacket->w,
@@ -379,8 +365,10 @@ void ProcessMessage(unsigned long type,unsigned long *body)
     break;
 
   case M_DESTROY_WINDOW:
-    if ((i = DeleteItem(&windows, body[0])) == -1) break;
-    if (FindItem(&swallowed, body[0]) != -1) break;
+    if ((i = DeleteItem(&windows, body[0])) == -1)
+      break;
+    if (FindItem(&swallowed, body[0]) != -1)
+      break;
     RemoveButton(&buttons, i);
     redraw = 1;
     break;
@@ -692,7 +680,7 @@ static void ParseConfigLine(char *tline) {
   } else if(strncasecmp(tline,"ButtonWidth",11) == 0) {
     button_width = atoi(&tline[11]);
   } else if(strncasecmp(tline,"ImagePath",9) == 0) {
-    CopyString(&ImagePath, &tline[8]);
+    CopyString(&ImagePath, &tline[9]);
   } else {
     GoodiesParseConfig(tline, Module);
     StartButtonParseConfig(tline, Module);
@@ -736,19 +724,37 @@ void Swallow(unsigned long *body) {
 /******************************************************************************
   Alarm - Handle a SIGALRM - used to implement timeout events
 ******************************************************************************/
-void Alarm(int nonsense) {
-
-  switch(AlarmSet) {
+void Alarm(int nonsense)
+{
+  XEvent event;
+  Bool trigger_event = False;
 
+  switch(AlarmSet)
+  {
   case SHOW_TIP:
-    ShowTipWindow(1);
+    if (tip_window_alarm == False)
+    {
+      tip_window_alarm = True;
+      trigger_event = True;
+    }
     break;
 
   case HIDE_TASK_BAR:
-    HideTaskBar();
+    if (hide_taskbar_alarm == False)
+    {
+      hide_taskbar_alarm = True;
+      trigger_event = True;
+    }
     break;
   }
 
+  if (trigger_event)
+  {
+    event.xmotion.x = -1;
+    event.xmotion.y = -1;
+    event.xany.type = MotionNotify;
+    XSendEvent(dpy, win, False, MotionNotify, &event);
+  }
   AlarmSet = NOT_SET;
   signal (SIGALRM, Alarm);
 }
@@ -816,6 +822,12 @@ void LoopOnEvents()
   while(XPending(dpy)) {
     redraw = -1;
     XNextEvent(dpy, &Event);
+    if (Event.xany.type == ConfigureNotify)
+    {
+      /* Purge all but the last configure events */
+      while(XCheckTypedEvent(dpy, ConfigureNotify, &Event))
+	;
+    }
 
     switch(Event.type) {
       case ButtonRelease:
@@ -881,7 +893,7 @@ void LoopOnEvents()
           if (Event.xexpose.window == Tip.win)
             redraw = 0;
           else
-	      redraw = 1;
+	    redraw = 1;
 	}
         break;
 
@@ -942,6 +954,21 @@ void LoopOnEvents()
         break;
 
       case MotionNotify:
+	if (Event.xmotion.x < 0 && Event.xmotion.y < 0)
+	{
+	  /* This condition means that the event was triggered by an Alarm */
+	  if (hide_taskbar_alarm == True)
+	  {
+	    hide_taskbar_alarm = False;
+	    HideTaskBar();
+	  }
+	  else if (tip_window_alarm == True)
+	  {
+	    tip_window_alarm = False;
+	    ShowTipWindow(1);
+	  }
+	  break;
+	}
 	if (MouseInStartButton(Event.xmotion.x, Event.xbutton.y)) {
 	  if (SomeButtonDown(Event.xmotion.state))
 	    redraw = StartButtonUpdate(NULL, BUTTON_DOWN) ? 0 : -1;
@@ -970,7 +997,8 @@ void LoopOnEvents()
         if ((Event.xconfigure.width != win_width ||
 	     Event.xconfigure.height != win_height)) {
 	  AdjustWindow(Event.xconfigure.width, Event.xconfigure.height);
-          if (AutoStick) WarpTaskBar(win_y);
+          if (AutoStick)
+	    WarpTaskBar(win_y);
 	  redraw = 1;
         }
         else if (Event.xconfigure.x != win_x || Event.xconfigure.y != win_y) {
@@ -1416,7 +1444,8 @@ void WarpTaskBar(int y) {
     win_y = (int)ScreenHeight - win_height - win_border;
 
   XMoveWindow(dpy, win, win_x, win_y);
-  if (AutoHide) SetAlarm(HIDE_TASK_BAR);
+  if (AutoHide)
+    SetAlarm(HIDE_TASK_BAR);
 
   /* Prevent oscillations caused by race with
      time delayed TaskBarHide().  Is there any way
