@@ -56,7 +56,7 @@
 #include <unistd.h>
 #include <ctype.h>
 #include <stdlib.h>
-#include "fvwm/module.h"
+#include "libs/Module.h"
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xproto.h>
@@ -105,7 +105,8 @@ int PlayerChannel[2];
 char *MyName;
 
 Display *dpy;
-int x_fd,fd_width;
+int x_fd;
+fd_set_size_t fd_width;
 int ROWS = FALSE;
 
 Window Root;
@@ -1679,7 +1680,7 @@ void ParseOptions(char *filename)
 	}
       else if((strlen(&tline[0])>1)&&(strncasecmp(tline,"ImagePath",9)==0))
 	{
-	  CopyString(&imagePath,&tline[8]);
+	  CopyString(&imagePath,&tline[9]);
 	}
 #ifdef ENABLE_SOUND
       else if((strlen(&tline[0])>1)&&(strncasecmp(tline,"*AudioDir",9)==0))
@@ -2034,11 +2035,7 @@ int My_XNextEvent(Display *dpy, XEvent *event)
   FD_SET(x_fd,&in_fdset);
   FD_SET(fd[1],&in_fdset);
 
-#ifdef __hpux
-  select(fd_width,(int *)&in_fdset, 0, 0, NULL);
-#else
-  select(fd_width,&in_fdset, 0, 0, NULL);
-#endif
+  select( fd_width, SELECT_FD_SET_CAST &in_fdset, 0, 0, NULL );
 
 
   if(FD_ISSET(x_fd, &in_fdset))
